//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================


#ifndef CPSYMUDAPPLICATION_H
#define CPSYMUDAPPLICATION_H

#include <sys/time.h>
#include <string>
#include <exception>
//#include <map>
#include <vector>
//#include <list>
#include "CPsyMudLog.h"
#include "CPsyMudComm.h"
#include "CPsyMudPCharacter.h"
#include "CPsyMudUniverse.h"

//#define MAX_PLAYLIST_ENTRIES 				300		// nuke later when changing playlist to map/vector etc.

#define MICROSECONDS_PER_PULSE				100000					// 10 pulses per second
#define PULSES_PER_SECOND					(1000000 / MICROSECONDS_PER_PULSE)

#define REAL_SECONDS_PER_MUD_HOUR			30
#define PULSES_BETWEEN_ZONE_UPDATES			(10 * PULSES_PER_SECOND)
#define PULSES_BETWEEN_NPC_UPDATES			(10 * PULSES_PER_SECOND)
#define PULSES_BETWEEN_FIGHT_UPDATES		(2 * PULSES_PER_SECOND)
#define PULSES_BETWEEN_TIME_UPDATES			(REAL_SECONDS_PER_MUD_HOUR * PULSES_PER_SECOND)
#define PULSES_BETWEEN_WEATHER_UPDATES		(REAL_SECONDS_PER_MUD_HOUR * PULSES_PER_SECOND)
#define PULSES_BETWEEN_POINT_UPDATES		(REAL_SECONDS_PER_MUD_HOUR * PULSES_PER_SECOND)
#define PULSES_BETWEEN_SPELL_AFFECT_UPDATES	(REAL_SECONDS_PER_MUD_HOUR * PULSES_PER_SECOND)
#define PULSES_BETWEEN_CRASH_SAVES			(60 * PULSES_PER_SECOND)
#define PULSES_BETWEEN_USAGE_UPDATE			(5 * 60 * PULSES_PER_SECOND)

#define PULSES_BETWEEN_PULSE_ROLLOVER		(600 * 60 * PULSES_PER_SECOND)



class CPsyMudAppException : public exception
{
private:
	string sLocation ;		// locatation of error
	string sErrorString ;	// description of exception
public:
	CPsyMudAppException(char * szLocation, char * szWhat)
				: sLocation(szLocation), sErrorString(szWhat) {}

	virtual const char *what() const
	{
		return sErrorString.c_str() ;
	}
	
	const char *where() const
	{
		return sLocation.c_str() ;
	}

} ;




class CPsyMudApp
{
private:
	unsigned int 	m_uiPort ;				// port we're listening to
	bool 			m_bRestricted ;			// restricted to only existing players
	short int 		m_iMinLevel ;			// minimum level required to login
	
	int				m_iCurrentPulse ;		// current pulse in the game, lowest level
	
	CPsyMudComm 	m_comm ;					// communications with outside world
	CPsyMudUniverse	m_Universe ;			// the database, files etc.

	vector<CPsyMudPCharacter *> 			m_playlist ;

	void AcceptNewConnections() ;
	void IncrementPulse() ;
	void ProcessPendingInput() ;
	void ProcessPendingOutput()	;
	void DisconnectUsers() ;
	void PauseExecution(struct timeval& tmPauseDuration) ;

	void InterpretInput() ;					// calls the interpreter for all input for all characters

	void HealCharacters() ;

	void PerformHeartbeatProcessing() ;

	void GameLoop() ;

	void NewCharacterSetup(CPsyMudPCharacter * pch) ;

public:
	CPsyMudApp() ;
	~CPsyMudApp() ;

	const vector<CPsyMudPCharacter *> * GetActivePlayerList() const		{ return &m_playlist ; }
	inline CPsyMudUniverse *	GetUniverse() 	{ return &m_Universe ; }	


	void SetPort(const unsigned int iNewPort) ;
	void SetRestricted(const bool bRestricted) ;
	void SetUniverseDirectory(const char * szUniverseDirectory) throw(CPsyMudAppException) ;
	
	void Run() ;							// starts it all

	void RoomMessage(const string& sMessage, CPsyMudRoom * room, CPsyMudPCharacter * pchExclude = NULL) ;
	//void ZoneMessage(const string& sMessage, CPsyMudZone * zone, CPsyMudPCharacter * pchExclude = NULL) ;
	//void WorldMessage(const string& sMessage, CPsyMudWorld * world, CPsyMudPCharacter * pchExclude = NULL) ;
	void UniverseMessage(const string& sMessage, CPsyMudPCharacter * pchExclude = NULL) ;

} ;

extern CPsyMudApp	g_Mud ;

#endif // ifndef CPSYMUDAPPLICATION_H
