//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================


#include <iostream>
#include "CPsyMudBuffer.h"
#include "stringutil.h"

#define	COMMAND_LINE_SEPARATOR 	'\xfe'		// just some dummy character we're unlikely to see

string CPsyMudBuffer::SubstituteSeparators(const string& sTextToProcess)
{
	string 	sProcessed ;

	//cerr << "DEBUG : processing text [" << sTextToProcess.c_str() << "]" << endl ;

	//cerr << "   --- m_bIsInputBuffer == " << m_bIsInputBuffer << endl ;

	if (m_bIsInputBuffer)
	{
		bool	bLastCharacterWasEOL = false ;

		for (string::const_iterator p = sTextToProcess.begin() ; p < sTextToProcess.end() ; p++)
		{
			if ((*p) == '\r' || (*p) == '\n')
			{
				m_bHasCommand = true ;

				if (!bLastCharacterWasEOL)
				{
					sProcessed += COMMAND_LINE_SEPARATOR ;
				}

				bLastCharacterWasEOL = true ;
			}
			else
			{
				sProcessed += *p ;
				bLastCharacterWasEOL = false ;
			}
		}
	}	
	else
	{
		sProcessed = sTextToProcess ;
	}

	//cerr << "   --- m_bHasCommand == " << m_bHasCommand << endl ;
	
	return sProcessed ;
}

CPsyMudBuffer::CPsyMudBuffer(const int iDefaultBufferSize, const bool bIsInputBuffer)
{
	m_sBuffer.reserve(iDefaultBufferSize) ;
	m_bIsInputBuffer = bIsInputBuffer ;
	m_bHasCommand = false ;
}

CPsyMudBuffer::CPsyMudBuffer(const CPsyMudBuffer& copy)
{
	m_bHasCommand = copy.m_bHasCommand ;
	m_sBuffer = copy.m_sBuffer ;
	m_bIsInputBuffer = copy.m_bIsInputBuffer ;
}


CPsyMudBuffer::~CPsyMudBuffer()
{
	m_sBuffer.erase(m_sBuffer.begin(), m_sBuffer.end()) ;
}

void CPsyMudBuffer::AppendToEnd(const string& sTextToAppend)
{	
	m_sBuffer += SubstituteSeparators(sTextToAppend) ;
}

void CPsyMudBuffer::AppendToBeginning(const string& sTextToAppend)
{
	m_sBuffer = SubstituteSeparators(sTextToAppend) + m_sBuffer ;
}

string CPsyMudBuffer::RemoveAll()
{
	string sTmp ;

	sTmp = m_sBuffer ;

	m_sBuffer = "" ;
	m_bHasCommand = false ;

	return sTmp ;
}

string CPsyMudBuffer::RemoveFirstCommandLine()
{
	string sCommandLine ;
	string sLeftover ;

	SplitString(sCommandLine, sLeftover, m_sBuffer, COMMAND_LINE_SEPARATOR) ;

	m_bHasCommand = false ;

	m_sBuffer = sLeftover ;

	for (string::const_iterator p = m_sBuffer.begin() ; p < m_sBuffer.end() && !m_bHasCommand ; p++)
	{
		if (*p == COMMAND_LINE_SEPARATOR)
		{
			m_bHasCommand = true ;
		}
	}

	return sCommandLine ;
}
