//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================

#ifndef CPSYMUDBUFFER_H
#define CPSYMUDBUFFER_H

#include <string>

class CPsyMudBuffer
{
private:
	bool 	m_bHasCommand ;
	bool	m_bIsInputBuffer ;		// true if we should process newlines.  Only good for input buffers.
	string	m_sBuffer ;

	string	SubstituteSeparators(const string& sTextToProcess) ;

public:
	CPsyMudBuffer(const int iDefaultBufferSize, const bool bIsInputBuffer) ;
	CPsyMudBuffer(const CPsyMudBuffer& copy) ;
	~CPsyMudBuffer() ;

	void		AppendToEnd(const string& sTextToAppend) ;
	inline void	AppendToEnd(const char * szTextToAppend) 			{ AppendToEnd(string(szTextToAppend)) ; }

	void		AppendToBeginning(const string& sTextToAppend) ;
	inline void	AppendToBeginning(const char * szTextToAppend) 		{ AppendToBeginning(string(szTextToAppend)) ; }

	string 		RemoveFirstCommandLine() ;
	string		RemoveAll() ;

	inline void			Clear() 			{ m_sBuffer.erase(m_sBuffer.begin(), m_sBuffer.end()) ; m_bHasCommand = false ; }
	inline const long	Length() const		{ return m_sBuffer.length() ; }

	inline bool			HasCommand() const	{ return m_bHasCommand ; }
} ;

#endif		//#define CPSYMUDBUFFER_H