//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================


#include "CPsyMudCharacter.h"

const char * CPsyMudCharPoints::GetHealthConditionDescription() const
{
	short int iHealthPercent = GetHitPointsPercentage() ;

	switch (iHealthPercent)
	{
		case 0 ... 5 :		return "is about to die" ;
		case 6 ... 10 :		return "is looking at the pretty white light" ;
		case 11 ... 15 :	return "looks like the meat room floor at Food Dog" ;
		case 16 ... 20 :	return "looks like the meat room floor at Food Dog" ;
		case 21 ... 25 :	return "is barely holding together" ;
		case 26 ... 30 :	return "is barely holding together" ;
		case 31 ... 35 :	return "is covered with their own blood" ;
		case 36 ... 40 :	return "is covered with their own blood" ;
		case 41 ... 45 :	return "is covered with their own blood" ;
		case 46 ... 50 :	return "is really hurting" ;
		case 51 ... 55 :	return "is really hurting" ;
		case 56 ... 60 :	return "is really hurting" ;
		case 61 ... 65 :	return "is bleeding from some nasty open wounds" ;
		case 66 ... 70 :	return "is bleeding from some nasty open wounds" ;
		case 71 ... 75 :	return "looks a bit like Rocky after a fight" ;
		case 76 ... 80 :	return "looks a bit like Rocky after a fight" ;
		case 81 ... 85 :	return "is bruised and scraped up a bit" ;
		case 86 ... 90 :	return "is bruised and scraped up a bit" ;
		case 91 ... 95 :	return "has a couple scrapes" ;
		case 96 ... 100 :	return "is in perfect condition" ;
		default :
			return "ERROR: health percent not in range 0 to 100." ;
	}
}

const string CPsyMudCharacter::GetPositionDescription()
{
	switch (m_position)
	{
		case cpDead :
			return string("dead") ;
		case cpSleeping :
			return string("sleeping") ;		// TO DO: return what they are sleeping on
		case cpResting :
			return string("resting") ;		// TO DO: return what they are resting on
		case cpSitting :
			return string("sitting") ;		// TO DO: return what they are sitting on
		case cpStanding :
			return string("standing") ;
		case cpFighting :
			return string("fighting") ;		// TO DO: return what they are fighting
		default :
			return string("unknown position") ;
	}
}


// called on "heal" ticks
string CPsyMudCharacter::NaturallyHealPoints()
{
	if (m_points.NeedHealing())
	{
		// lets figure out how we're going to heal this person

		short int	iHitHealAmount = 0 ;
		short int	iManaHealAmount = 0 ;
		short int	iMovementHealAmount = 0 ;

		// first, figure out base heal amounts as a function of
		// level, stats etc

		// add bonuses for position

		if (IsSleeping())
		{
			iHitHealAmount += (m_points.GetMaxHitPoints() / 10) + 3 ;
			iManaHealAmount += (m_points.GetMaxMana() / 10) + 3 ;
			iMovementHealAmount += (m_points.GetMaxMovementPoints() / 10) + 3 ;
		}
		else if (IsResting())
		{
			iHitHealAmount += (m_points.GetMaxHitPoints() / 13) + 2 ;
			iManaHealAmount += (m_points.GetMaxMana() / 13) + 2 ;
			iMovementHealAmount += (m_points.GetMaxMovementPoints() / 13) + 2 ;
		}
		else if (IsSitting())
		{
			iHitHealAmount += (m_points.GetMaxHitPoints() / 15) + 2 ;
			iManaHealAmount += (m_points.GetMaxMana() / 15) + 2 ;
			iMovementHealAmount += (m_points.GetMaxMovementPoints() / 15) + 2 ;
		}
		else if (IsStanding())
		{
			iHitHealAmount += (m_points.GetMaxHitPoints() / 17) + 1 ;
			iManaHealAmount += (m_points.GetMaxMana() / 17) + 1 ;
			iMovementHealAmount += (m_points.GetMaxMovementPoints() / 17) + 1 ;
		}
		else if (IsFighting())
		{
			iHitHealAmount += (m_points.GetMaxHitPoints() / 20) + 1 ;
			iManaHealAmount += (m_points.GetMaxMana() / 20) + 1 ;
			iMovementHealAmount += (m_points.GetMaxMovementPoints() / 20) + 1 ;
		}
		else
		{
			iHitHealAmount += 1 ;
			iManaHealAmount += 1 ;
			iMovementHealAmount += 1 ;
		}

		// add bonuses for race

		// add bonuses for class
	
		// now, add any bonuses for equipment, subtract for good eq on vampire, poison etc.


		// finally, subtract if they are tired, hungry etc.

		// now apply the heal

		m_points.HealHitPointsBy(iHitHealAmount) ;
		m_points.HealManaPointsBy(iManaHealAmount) ;
		m_points.HealMovementPointsBy(iMovementHealAmount) ;

		// now, return a string that tells them how they feel

		if (iHitHealAmount > 0)
		{
			return string("You feel better.\r\n") ;
		}
		else if (iHitHealAmount < 0)
		{
			return string("You feel worse.\r\n") ;
		}
		else
		{
			return string("") ;
		}
	}
	else
	{
		return string("") ;
	}
}