//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================



#ifndef CPSYMUDCHARACTER_H
#define CPSYMUDCHARACTER_H

#include <string>
#include "CPsyMudObject.h"
#include "CPsyMudItem.h"
#include "CPsyMudSkill.h"
#include "CPsyMudRoom.h"
#include "CPsyMudObjectContainer.h"

const int 			MAX_CHARACTER_SPELLS_OR_SKILLS = 200 ;
const signed int	ALIGNMENT_GOOD = 300 ;
const signed int	ALIGNMENT_BAD = -300 ;
const signed int	ALIGNMENT_MAX = 1000 ;
const signed int	ALIGNMENT_MIN = -1000 ;


// =======================================================================

class CPsyMudCharPoints
{
private:

	signed long 	m_lExperience ;			
	signed int		m_iCurrentMana ;
	signed int 		m_iMaxMana ;
	signed int 		m_iCurrentHitPoints ;
	signed int		m_iMaxHitPoints ;
	signed int		m_iCurrentMovementPoints ;
	signed int		m_iMaxMovementPoints ;
	signed int		m_iAlignment ;				// -1000 to + 1000

public:
	inline CPsyMudCharPoints()
	{
		m_lExperience = 0 ;			m_iCurrentMana = 0 ; 	m_iMaxMana = 0 ;
		m_iCurrentHitPoints = 0 ;	m_iMaxHitPoints = 0 ;	m_iCurrentMovementPoints = 0 ;
		m_iMaxMovementPoints = 0 ;	m_iAlignment = 0 ;
	}

	
	inline void SetAlignment(const signed int i) 				{ m_iAlignment = (i > ALIGNMENT_MAX ? ALIGNMENT_MAX : ( i < ALIGNMENT_MIN ? ALIGNMENT_MIN : i)) ;  }
	inline void SetExperience(const signed long l)				{ m_lExperience = l ; }
	inline void SetCurrentMana(const signed int i)				{ m_iCurrentMana = i ; }
	inline void SetMaxMana(const signed int i)					{ m_iMaxMana = i ; }
	inline void SetCurrentHitPoints(const signed int i)			{ m_iCurrentHitPoints = i ; }
	inline void SetMaxHitPoints(const signed int i)				{ m_iMaxHitPoints = i ; }
	inline void SetCurrentMovementPoints(const signed int i)	{ m_iCurrentMovementPoints = i ; }
	inline void SetMaxMovementPoints(const signed int i)		{ m_iMaxMovementPoints = i ; }


	inline const signed int		GetAlignment() const				{ return m_iAlignment ; }	
	inline const signed long	GetExperience()	const				{ return m_lExperience ; }	
	inline const signed int		GetCurrentMana() const				{ return m_iCurrentMana ; }	
	inline const signed int 	GetMaxMana() const					{ return m_iMaxMana ; }	
	inline const signed int	 	GetCurrentHitPoints() const			{ return m_iCurrentHitPoints ; }	
	inline const signed int	 	GetMaxHitPoints() const				{ return m_iMaxHitPoints ; }	
	inline const signed int 	GetCurrentMovementPoints() const	{ return m_iCurrentMovementPoints ; }	
	inline const signed int		GetMaxMovementPoints() const		{ return m_iMaxMovementPoints ; }	

	inline const bool 			IsGoodAlignment() const				{ return m_iAlignment >= ALIGNMENT_GOOD ; }
	inline const bool 			IsBadAlignment() const				{ return m_iAlignment <= ALIGNMENT_BAD ; }
	inline const bool 			IsNeutralAlignment() const			{ return (m_iAlignment > ALIGNMENT_BAD) && (m_iAlignment < ALIGNMENT_GOOD) ; }

	inline const short int  GetHitPointsPercentage() const			{ return (short int)((m_iCurrentHitPoints / (float)m_iMaxHitPoints) * 100) ; }
	inline const short int  GetManaPointsPercentage() const			{ return (short int)((m_iCurrentMana / (float)m_iMaxMana) * 100) ; }
	inline const short int  GetMovementPointsPercentage() const		{ return (short int)((m_iCurrentMovementPoints / (float)m_iMaxMovementPoints) * 100) ; }

	const char *			GetHealthConditionDescription() const ;

	inline void 			HealHitPointsBy(const int i)			{ m_iCurrentHitPoints = m_iCurrentHitPoints + i > m_iMaxHitPoints ? m_iMaxHitPoints : m_iCurrentHitPoints + i ; }
	inline void 			HealManaPointsBy(const int i)			{ m_iCurrentMana = m_iCurrentMana + i > m_iMaxMana ? m_iMaxMana : m_iCurrentMana + i ; }
	inline void 			HealMovementPointsBy(const int i)		{ m_iCurrentMovementPoints = m_iCurrentMovementPoints + i > m_iMaxMovementPoints ? m_iMaxMovementPoints : m_iCurrentMovementPoints + i ; }

	inline const bool		NeedHealing() const						{ return m_iCurrentHitPoints < m_iMaxHitPoints || m_iCurrentMana < m_iMaxMana || m_iCurrentMovementPoints < m_iMaxMovementPoints ; }

} ;

// =======================================================================

class CPsyMudCharAbilities
{
private:
	signed char 	m_chStrength ;			
	signed char		m_chStrengthAdd ;		// 00 to 100 if strength is 18, ie 18/73
	signed char 	m_chIntelligence ;
	signed char 	m_chDexterity ;
	signed char		m_chWisdom ;
	signed char		m_chConstitution ;
	signed char		m_chCharisma ;

public:
	inline CPsyMudCharAbilities()
	{
		m_chStrength = 0 ;	m_chStrengthAdd = 0 ;	m_chIntelligence = 0 ;
		m_chDexterity = 0 ; m_chWisdom = 0 ; 		m_chConstitution = 0 ;	m_chCharisma = 0 ;
	}
	inline void SetStrength(const signed char ch) 		{ m_chStrength = ch ; 		return ; }
	inline void SetStrengthAdd(const signed char ch) 	{ m_chStrengthAdd = ch ; 	return ; }
	inline void SetIntelligence(const signed char ch) 	{ m_chIntelligence = ch ; 	return ; }
	inline void SetDexterity(const signed char ch) 		{ m_chDexterity = ch ; 		return ; }
	inline void SetWisdom(const signed char ch) 		{ m_chWisdom = ch ; 		return ; }
	inline void SetConstitution(const signed char ch) 	{ m_chConstitution = ch ; 	return ; }
	inline void SetCharisma(const signed char ch) 		{ m_chCharisma = ch ; 		return ; }
	
	inline const signed char GetStrength() const		{ return m_chStrength ; }
	inline const signed char GetStrengthAdd() const 	{ return m_chStrengthAdd ; }
	inline const signed char GetIntelligence() const	{ return m_chIntelligence ; }
	inline const signed char GetDexterity() const 		{ return m_chDexterity ; }
	inline const signed char GetWisdom() const 			{ return m_chWisdom ; }
	inline const signed char GetConstitution() const 	{ return m_chConstitution ; }
	inline const signed char GetCharisma() const 		{ return m_chCharisma ; }
} ;

// =======================================================================

class CPsyMudCharEquipment
{
private:
	
	CPsyMudItem *		m_LeftHandHoldOrWield ;		// sword, orb, light etc.
	CPsyMudItem *		m_RightHandHoldOrWield ;	// sword, orb, light etc.
	CPsyMudItem *		m_ShieldWear ;				// shield
	CPsyMudItem *		m_HeadWear ;				// hat, helm etc.
	CPsyMudItem *		m_EyeWear ;					// glasses, eyepatch etc.
	CPsyMudItem *		m_LeftEarWear ;				// earring, listening device etc.
	CPsyMudItem *		m_RightEarWear ;			// earring, listening device etc.
	CPsyMudItem *		m_FaceWear ;				// over face.  Mask, face paint etc.
	CPsyMudItem *		m_NoseWear ;				// nose ring etc.
	CPsyMudItem *		m_MouthWear ;				// mouthguard etc.
	CPsyMudItem *		m_NeckWear1 ;				// necklace, scarf etc.
	CPsyMudItem *		m_NeckWear2 ;				// necklace, scarf etc.
	CPsyMudItem *		m_OverBodyWear ;			// cape, cloak etc.
	CPsyMudItem *		m_LeftShoulderWear ;		// over-the-shoulder cannon, armor plate etc.
	CPsyMudItem *		m_RightShoulderWear ;		// over-the-shoulder cannon, armor plate etc.
	CPsyMudItem *		m_TorsoWear ;				// breastplate etc.
	CPsyMudItem *		m_WaistWear ;				// belt etc.
	CPsyMudItem *		m_CrotchWear ;				// belt etc.
	CPsyMudItem *		m_LeftWristWear ;			// watch, bracelet, etc.
	CPsyMudItem *		m_RightWristWear ;			// watch, bracelet, etc.
	CPsyMudItem *		m_HandsWear ;				// gloves, guantlets etc.
	CPsyMudItem *		m_LegsWear ;				// pants etc.
	CPsyMudItem *		m_FeetWear ;				// shoes, boots etc.
		
	CPsyMudItem *		m_RingFingerWear ;			// wedding ring etc.
	CPsyMudItem *		m_RingWear1 ;				// ring
	CPsyMudItem *		m_RingWear2 ;				// ring
	CPsyMudItem *		m_RingWear3 ;				// ring
	CPsyMudItem *		m_RingWear4 ;				// ring
	
	CPsyMudItem *		m_BadgeWear ;				// police badge, clan badge etc.

public:
	bool WearOrWieldObject () ;

} ;


// =======================================================================

class CPsyMudCharAttributes
{
private:
	unsigned int	m_uiHeight ;
	unsigned int	m_uiWeight ;
	unsigned int	m_uiWidth ;	
	string			m_sName ;
	string			m_sTitle ;	

public:

} ;

// =======================================================================

class CPsyMudCharSkillLevel
{
private:
	CPsyMudSkill *			m_skl ;						// the spell or skill
	signed char				m_chSkillLevelPercent ;		// how much they have learned.  000% to 100%
	unsigned long			m_ulNumSuccessfulUses ;		// how many times they have successfully used this spell/skill
	unsigned long			m_ulNumFailedUses ;			// how many times they have tried the spell and failed


public:
} ;

class CPsyMudCharSkillList
{
private:
	CPsyMudCharSkillLevel 	m_skills [MAX_CHARACTER_SPELLS_OR_SKILLS] ;

public:
	// GetSkillOrSpellByName(const char * szName) ;

} ;

// =======================================================================

class CPsyMudCharacter : public CPsyMudObject
{
public:
	enum PositionEnum
	{
		cpDead,
		cpSleeping,
		cpResting,
		cpSitting,
		cpStanding,
		cpFighting
	} ;

private:
	string				m_sName ;
	PositionEnum		m_position ;
	CPsyMudRoom *		m_Location ;

	//CPsyMudCharSkillList	m_skl ;
	//CPsyMudCharAbilities	m_abil ;
	CPsyMudCharPoints		m_points ;				// hitpoints, exp, etc			

	CPsyMudCharAttributes	m_attr ;			

	//CPsyMudCharEquipment	m_eq ;
	CPsyMudItemContainer	m_inv ;					// inventory of items (not worn equipment)

public:
	CPsyMudCharacter()
	{
		m_points.SetMaxHitPoints(50) ;
		m_points.SetMaxMana(90) ;
		m_points.SetMaxMovementPoints(100) ;
		m_sName = "NoName" ;
		m_position = cpStanding ;
	}

	CPsyMudCharacter(const CPsyMudCharacter& copy) : CPsyMudObject(copy)	{}
	~CPsyMudCharacter() {}

	inline const PositionEnum	GetPosition() const 		{ return m_position ; }
	inline void	SetPosition(const PositionEnum newpos)	{ m_position = newpos ; }

	inline	bool	IsDead()			{ return m_position == cpDead ; }
	inline	bool	IsSleeping()		{ return m_position == cpSleeping ; }
	inline	bool	IsResting()			{ return m_position == cpResting ; }
	inline	bool	IsSitting()			{ return m_position == cpSitting ; }
	inline	bool	IsStanding()		{ return m_position == cpStanding ; }
	inline	bool	IsFighting()		{ return m_position == cpFighting ; }

	inline CPsyMudCharPoints * GetPoints()	{ return &m_points; }

	string			NaturallyHealPoints() ;		// called when normal ticks go by

	inline CPsyMudRoom * GetLocation()			{ return m_Location ; }
	inline void 	SetLocation(CPsyMudRoom * loc)	{ m_Location = loc ; }

	inline const 	string GetName() const			{ return m_sName ; }
	inline void 	SetName(const string sNewName)	{ m_sName = sNewName ; }

	virtual void 	SerializeIn(void * data) 		{}
	virtual void 	SerializeOut(void * data) 		{}

	const string	GetPositionDescription() ;

	inline CPsyMudItemContainer *	GetInventory()		{ return &m_inv ; }
	

} ;


#endif 	// #ifndef CPSYMUDCHARACTER_H