//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================

#include <string>

#include "CPsyMudLog.h"
#include "CPsyMudCmdAlias.h"
#include "CPsyMudPCharacter.h"	// for alias stuff

#include "color.h"
#include "stringutil.h"


CPsyMudCmdAlias::CPsyMudCmdAlias()
		: CPsyMudCommand("alias", "Utility", "usage: alias [<command> [<definition>]]\n\r       List aliases, remove an alias or add an alias.\n\r")
{
}

void CPsyMudCmdAlias::ProcessCommand(const string& sRestOfCommandLine, CPsyMudPCharacter * pch)
{
	string sMsg ;

	if (sRestOfCommandLine == "")
	{
		// list aliases

		if (pch->GetAliases()->size() == 0)
		{
			sMsg = "You currently have no aliases defined.\r\n" ;
		}
		else
		{
			sMsg = COLOR_DARK_CYAN + string("Currently Defined Aliases\r\n") + COLOR_NORMAL ;

			for (vector<CPsyMudCommandAlias>::const_iterator p = pch->GetAliases()->begin() ; p < pch->GetAliases()->end() ; p++)
			{
				sMsg += string("  ") + COLOR_DARK_YELLOW + p->GetAlias() + COLOR_BRIGHT_YELLOW + " = " + COLOR_NORMAL + p->GetDefinition() + "\r\n" ;
			}

		}
	}
	else
	{
		string sAlias ;
		string sDefinition ;

		SplitString(sAlias, sDefinition, sRestOfCommandLine, ' ') ;

		if (sDefinition == "")
		{
			// did not provide a definition.  we're deleting this alias

			if (pch->RemoveAlias(sAlias))
			{
				sMsg = "Alias \"" + sAlias + "\" removed.\r\n" ;
			}
			else
			{
				sMsg = "Alias \"" + sAlias + "\" not found.\r\n" ;				
			}
		}
		else
		{
			// setting up a new alias
			if (pch->GetAliasDefinition(sAlias) == "")
			{
				pch->AddAlias(sAlias, sDefinition) ;
				sMsg = "Alias \"" + sAlias + "\" added.\r\n" ;				
			}
			else
			{
				sMsg = "Alias \"" + sAlias + "\" already exists.\r\n" ;				
			}
		}
	}
	pch->GetConnectionInfo()->AddToOutputBuffer(sMsg) ;
}
