//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================


#include "CPsyMudCmdGo.h"
#include "CPsyMudLog.h"
#include "CPsyMudCmdLook.h"

CPsyMudCmdGo::CPsyMudCmdGo()
		: CPsyMudCommand("go", "Movement", "usage: go [direction]\n\r       Attempt to move your character in the specified direction.\n\r")
{
}

void CPsyMudCmdGo::ProcessCommand(const string& sRestOfCommandLine, CPsyMudPCharacter * pch)
{
	if (pch->IsSleeping())
	{
		pch->GetConnectionInfo()->AddToOutputBuffer("You're not very good at sleepwalking.\r\n") ;
	}
	else if (pch->IsResting())
	{
		pch->GetConnectionInfo()->AddToOutputBuffer("You feel too relaxed for that.\r\n") ;
	}
	else if (pch->IsSitting())
	{
		pch->GetConnectionInfo()->AddToOutputBuffer("You realize it's too hard to do when sitting down.\r\n") ;
	}
	else if (pch->IsFighting())
	{
		pch->GetConnectionInfo()->AddToOutputBuffer("You can't just waltz out of a fight!.\r\n") ;
	}
	else
	{
		if (sRestOfCommandLine == "")
		{
			pch->GetConnectionInfo()->AddToOutputBuffer("Go where?\r\n") ;
		}
		else
		{
			static string sMsg ;		// static is nice because it gives us limited scope, without the stack overhead
	
			if (!pch->GetLocation())
			{
				g_Log.Log(CPsyMudLog::logError, "Ack!  Character's location was NULL!") ;
				pch->GetConnectionInfo()->AddToOutputBuffer("Woah! You're currently not located anywhere.  That's a problem.  Contact someone about it.") ;
			}
			else
			{
				CPsyMudRoomExit * 	ex ;

				ex = pch->GetLocation()->GetExitByName(sRestOfCommandLine.c_str()) ;

				if (ex)
				{
					if (ex->GetDestination())
					{

						// display exit message to this user

						// display exit message to other users

						// move player to linked room
						pch->SetLocation((CPsyMudRoom *)(ex->GetDestination())) ;

						// display entrance message to this user

						// display entrance message to this user

						// if character has "verbose" on, we show them the description of the next room
						static CPsyMudCmdLook look ;

						if (pch->GetVerboseMode())
						{
							// easiest way to handle this in a nice encapsulated way is just to execute the look
							// command with no additional parameters (look at current room)
							look.ProcessCommand(string(""), pch) ;
						}
					}
					else
					{
						pch->GetConnectionInfo()->AddToOutputBuffer("BUG! That exit doesn't go anywhere.\r\n") ;
						g_Log.Log(CPsyMudLog::logError, "Ack!  Player tried to go through an exit that goes nowhere.  Check log for unresolved room references during startup.") ;
					}
				}
				else
				{
					pch->GetConnectionInfo()->AddToOutputBuffer("You can't go that way.\r\n") ;
				}
			}
		}
	}
}