//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================


#include <stdio.h>		// for atol

#include "CPsyMudApp.h"
#include "CPsyMudRoom.h"
#include "CPsyMudCmdGoto.h"
#include "CPsyMudLog.h"
#include "CPsyMudCmdLook.h"

CPsyMudCmdGoto::CPsyMudCmdGoto()
		: CPsyMudCommand("goto", "Movement", "usage: goto [room number]\n\r       Go directly to the specified room (high level required.).\n\r")
{
}

void CPsyMudCmdGoto::ProcessCommand(const string& sRestOfCommandLine, CPsyMudPCharacter * pch)
{
	if (sRestOfCommandLine == "")
	{
		pch->GetConnectionInfo()->AddToOutputBuffer("Goto where?\r\n") ;
	}
	else
	{
		static string sMsg ;
		unsigned long ulRoomNumber = atol(sRestOfCommandLine.c_str()) ;
		

		if (ulRoomNumber > 0)
		{
			CPsyMudRoom * rm ;

			rm = g_Mud.GetUniverse()->GetRoomByNumber(ulRoomNumber) ;

			if (rm)
			{
				sMsg = "You are now in room \"" + rm->GetName() + "\"\r\n" ;

				pch->SetLocation(rm) ;
				pch->GetConnectionInfo()->AddToOutputBuffer(sMsg) ;
			}
			else
			{
				sMsg = "No room with that ID\r\n" ;
				pch->GetConnectionInfo()->AddToOutputBuffer(sMsg) ;
			}
		}
		else
		{
			pch->GetConnectionInfo()->AddToOutputBuffer("Please specify a valid, non-zero room number\r\n") ;
		}		
	}
}