//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================



#include "CPsyMudApp.h"
#include "CPsyMudCmdLook.h"
#include "CPsyMudLog.h"
#include "color.h"
#include "CPsyMudRoom.h"
#include "CPsyMudRoomExit.h"
#include <stdio.h>		// for sprintf

CPsyMudCmdLook::CPsyMudCmdLook()
		: CPsyMudCommand("look", "Environment", "usage: look [item | character | direction]\n\r       Look at the current room, an item character or direction.\n\r")
{
}

void CPsyMudCmdLook::ProcessCommand(const string& sRestOfCommandLine, CPsyMudPCharacter * pch)
{
	static string sMsg ;

	if (pch->IsSleeping())
	{
		pch->GetConnectionInfo()->AddToOutputBuffer("You see the inside of your eyelids.\r\n") ;
	}
	else
	{
		if (sRestOfCommandLine == "")
		{
			CPsyMudRoom * rm = pch->GetLocation() ;

			if (rm)
			{
				sMsg = "" ;

				// show room name				
				sMsg += COLOR_DARK_CYAN + string(rm->GetName()) + COLOR_NORMAL ;

				// if an imm, or if I own this room, show the room number
				// TO DO: Actual checks for ownership or imm status
				sMsg += COLOR_DARK_YELLOW + string(" [") + rm->GetObjectIDAsString() + string("] ") + COLOR_NORMAL ;

				sMsg += "\r\n" ;

				// show room description
				sMsg += string(rm->GetDescription()) + "\r\n" ;
				pch->GetConnectionInfo()->AddToOutputBuffer(sMsg) ;


				// show room exits

				if (pch->GetAutoExits())
				{
					static char 	szFormattedDirectionName[MAX_EXIT_DIRECTION_NAME_LEN+1] ;
					bool			bExitPrinted = false ;
					

					pch->GetConnectionInfo()->AddToOutputBuffer(COLOR_DARK_CYAN + string("\r\nObvious Exits\r\n") + COLOR_DARK_CYAN) ;						

					for (vector<CPsyMudRoomExit *>::const_iterator p = rm->GetExitList()->begin() ; p < rm->GetExitList()->end() ; p++)
					{
						// TO DO: Check to see if this exit is hidden or anything like that before
						// displaying it in the list.

						bExitPrinted = true ;

						sprintf(szFormattedDirectionName, "  %-15s", (*p)->GetDirectionName().c_str()) ;

						sMsg = COLOR_DARK_YELLOW + string(szFormattedDirectionName) + COLOR_NORMAL + string(" : ") + COLOR_DARK_CYAN + (*p)->GetDescription() + COLOR_NORMAL + string("\r\n") ;

						pch->GetConnectionInfo()->AddToOutputBuffer(sMsg) ;
					}

					if (!bExitPrinted)
					{
						pch->GetConnectionInfo()->AddToOutputBuffer("  (none)\r\n") ;						
					}
				}
				else
				{
					pch->GetConnectionInfo()->AddToOutputBuffer("(DEBUG: 'Autoexits' is off)\r\n") ;						
				}

				// show players in room
			
				// TEMP !!! This is far too inefficient for real use
				for (vector <CPsyMudPCharacter *>::const_iterator p = g_Mud.GetActivePlayerList()->begin() ; p != g_Mud.GetActivePlayerList()->end() ; p ++)
				{

					// see if player is in same room.  If so, display them
					if ((*p) != pch && (*p)->GetLocation() == pch->GetLocation())
					{
						sMsg = COLOR_BRIGHT_YELLOW + (*p)->GetName() + COLOR_NORMAL + string(" is ") + (*p)->GetPositionDescription() + " here.\r\n" ;
						pch->GetConnectionInfo()->AddToOutputBuffer(sMsg) ;

							// maybe show health state ?
						
					}	
				}
				// end temp!

				// show NPCs in the room

				// show items in the room
				if (pch->GetLocation()->GetItems()->Count() > 0)
				{
					pch->GetConnectionInfo()->AddToOutputBuffer(COLOR_DARK_CYAN + string("\r\nItems You See Here\r\n") + COLOR_DARK_CYAN) ;						
					for (list<CPsyMudItem *>::const_iterator p = rm->GetItems()->GetItemList()->begin() ; p != rm->GetItems()->GetItemList()->end() ; p++)
					{
						// TO DO: Check for invisible, hidden, whatever
	
						sMsg = string("  ") + (*p)->GetShortDescription() + COLOR_NORMAL + string("\r\n") ;
						pch->GetConnectionInfo()->AddToOutputBuffer(sMsg) ;
					}
				}

				// add a final blank line
				pch->GetConnectionInfo()->AddToOutputBuffer("\r\n") ;						

			}
			else
			{
				pch->GetConnectionInfo()->AddToOutputBuffer("Ack!  You are currently not located anywhere.  Bug!  You need to contact someone.\r\n") ;
				g_Log.Log(CPsyMudLog::logError, "Bug!  Player's location is NULL!") ;
			}
		}
		else
		{
			CPsyMudItem * itm ;

			// first check to see if item is in inventory.  If so, look at that

			if (itm = pch->GetInventory()->GetItemByName(sRestOfCommandLine))
			{
				sMsg = itm->GetLongDescription() ;
				if (sMsg == "")
				{
					sMsg = "You see nothing special about that item.\r\n" ;
				}
				else
				{
					sMsg += "\r\n" ;
				}

				sMsg = string(COLOR_DARK_CYAN) + "(You look at the item in your inventory)\r\n" + string(COLOR_NORMAL) + sMsg ;
			}
			// next check to see if "item" is really a room extra description.

			// next check to see if item is an item in the room.  if so, look at that.
			else if (itm = pch->GetLocation()->GetItems()->GetItemByName(sRestOfCommandLine))
			{
				sMsg = itm->GetLongDescription() ;
				if (sMsg == "")
				{
					sMsg = "You see nothing special about that item.\r\n" ;
				}
				else
				{
					sMsg += "\r\n" ;
				}
				sMsg = string(COLOR_DARK_CYAN) + "(You look at the item in your current location)\r\n" + string(COLOR_NORMAL) + sMsg ;
			}

			// finally, check to see if the item is a player or NPC in the room

			else	// nothing found
			{
				sMsg = "You don't see any " + sRestOfCommandLine + " here.\r\n" ;
			}

			pch->GetConnectionInfo()->AddToOutputBuffer(sMsg.c_str()) ;
		}
	}
}