//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================


#ifndef CPSYMUDCOMM_H
#define CPSYMUDCOMM_H

//#include <list>		// stl linked list
#include <vector>
#include <fcntl.h>
#include <stdlib.h>
#include <string>

#include "CPsyMudBuffer.h"
#include "stringutil.h"

#define MAX_HOSTNAME					255
#define	MAX_INPUT_LENGTH				255		// maximum input text length
#define DEFAULT_OUTPUT_BUFFER_SIZE		5020
#define DEFAULT_INPUT_BUFFER_SIZE		2048


class CPsyMudConnInfo
{
private:
	int		m_fd ;											// handle to socket file descriptor
	char 	m_szHostname[MAX_HOSTNAME+1] ;					// hostname the user came in from	
	CPsyMudBuffer	m_bufOutputBuffer ;						// outgoing buffer (to user)
	CPsyMudBuffer	m_bufInputBuffer ;						// incoming buffer (from user)

	bool	m_bNeedsToBeDisconnectedDueToCommError ;		// true if needs to be disconnected due to comm error
	bool	m_bNeedsToBeDisconnectedDueToQuit ;				// true if needs to be disconnected

	bool	m_bHasPendingCommandInInputBuffer ;				// true if there is at least one new line in input buffer

	bool	m_bHasPrompt ;									// true if they currently have the prompt displayed

	int 	WriteToSocket() ;
	int 	ReadFromSocket() ;

public:
	void CloseConnection() ;

	CPsyMudConnInfo(int sockfd) ;//, char * szHostName)
	~CPsyMudConnInfo() ;

	inline CPsyMudConnInfo(const CPsyMudConnInfo& copy) : m_bufOutputBuffer(copy.m_bufOutputBuffer), m_bufInputBuffer(copy.m_bufInputBuffer)
	{
		m_fd = copy.m_fd ;
		strncpy(m_szHostname, copy.m_szHostname, MAX_HOSTNAME) ;
		m_bNeedsToBeDisconnectedDueToCommError = copy.m_bNeedsToBeDisconnectedDueToCommError ;
		m_bNeedsToBeDisconnectedDueToQuit = copy.m_bNeedsToBeDisconnectedDueToQuit ;

		m_bHasPendingCommandInInputBuffer = copy.m_bHasPendingCommandInInputBuffer ;

		m_bHasPrompt = copy.m_bHasPrompt ;

		m_bufOutputBuffer = copy.m_bufOutputBuffer ;
		m_bufInputBuffer = copy.m_bufInputBuffer ;
	
	}
	void operator=(const CPsyMudConnInfo& rhs)
	{
		m_fd = rhs.m_fd ;
		strncpy(m_szHostname, rhs.m_szHostname, MAX_HOSTNAME) ;

		m_bNeedsToBeDisconnectedDueToCommError = rhs.m_bNeedsToBeDisconnectedDueToCommError ;
		m_bNeedsToBeDisconnectedDueToQuit = rhs.m_bNeedsToBeDisconnectedDueToQuit ;

		m_bHasPendingCommandInInputBuffer = rhs.m_bHasPendingCommandInInputBuffer ;

		m_bHasPrompt = rhs.m_bHasPrompt ;

		m_bufInputBuffer = rhs.m_bufInputBuffer ;
		m_bufOutputBuffer = rhs.m_bufOutputBuffer ;
	}

	bool operator== (const CPsyMudConnInfo& rhs) const		{ return (m_fd == rhs.m_fd) ; }
	bool operator< (const CPsyMudConnInfo& rhs) const		{ return (m_fd < rhs.m_fd) ; }

	inline void	SetSocketDescriptor(const int fd)			{ m_fd = fd ; }
	inline int	GetSocketDescriptor() const 				{ return m_fd ; }

	inline bool	NeedsToBeDisconnectedDueToCommError() const	{ return m_bNeedsToBeDisconnectedDueToCommError ; }

	inline bool	NeedsToBeDisconnectedDueToQuit() const		{ return m_bNeedsToBeDisconnectedDueToQuit ; }
	inline void	SetUserWantsToQuit(const bool bQuit)		{ m_bNeedsToBeDisconnectedDueToQuit = bQuit ; }

	inline bool	HasPendingCommandInInputBuffer() const		{ return m_bufInputBuffer.HasCommand() ; }

	inline bool	HasPendingOutput() const					{ return m_bufOutputBuffer.Length() > 0 ; }
	void 		AddToOutputBuffer(const string& sTextToOutput) ;
	inline void AddToOutputBuffer(const char * sTextToOutput) 	{ AddToOutputBuffer(string(sTextToOutput)) ; }


	void 		ProcessOutput(const string& sPrompt) ;

	inline bool	HasPendingInput() const				{ return m_bufInputBuffer.Length() > 0 ; }
	//void 		AddToInputBuffer(const char * szTextToInput) ;
	void 		ProcessInput() ;

	CPsyMudBuffer *	GetInputBuffer()				{ return &m_bufInputBuffer ; }
	CPsyMudBuffer *	GetOutputBuffer()				{ return &m_bufOutputBuffer ; }

	inline void ClearInputBuffer()					{ m_bufInputBuffer.Clear() ; }
	inline void ClearOutputBuffer()					{ m_bufOutputBuffer.Clear() ;	}

	string ChunkFirstCommandFromInputBuffer() ;

} ;


//====================================================================

class CPsyMudComm
{
private:
	unsigned int	m_iPort ;			// port we're listening to
	int				m_fdMother ;		// file descriptor for the server socket

	fd_set			m_InputSet ;		// descriptor for socket input "file"
	fd_set			m_OutputSet ;		// descriptor for socket output "file"
	fd_set			m_ExceptionSet ;	// descriptor for socket exception (out of band) "file"

	vector<CPsyMudConnInfo *>	m_lstConnections ;		// list of all the connections/descriptors.

	void 	MakeDescriptorNonBlocking(int sockfd) ;		// makes a file descriptor non-blocking using fcntl calls
	int		GetMaxDescriptor() ;						// gets the highest descriptor in the connection list

public:
	CPsyMudComm() ;
	~CPsyMudComm() ;

	void 	InitializeSocket() ;						// TO DO!! Add exception throwing here
	CPsyMudConnInfo * AcceptNewConnAndReturnInfo() ;	// and here

	inline void 			SetPort(const unsigned int iPort)	{ m_iPort = iPort ; }
	inline unsigned int 	GetPort() const 					{ return m_iPort ; }

	void	GoToSleep() ;
	void	ResetDescriptorSets() ;
	void	RemoveConnection(CPsyMudConnInfo & ci) ;
	void 	PollForActivity() ;

} ;



#endif	// #define CPSYMUDCOMM_H