//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================



#ifndef CPSYMUDCOMMAND_H
#define	CPSYMUDCOMMAND_H

#include <string>
#include <stdlib.h>
#include "CPsyMudPCharacter.h"

// class from which all commands are defined
class CPsyMudCommand
{
protected:
	string 		m_sCommandText ;		// actual text of command
	string 		m_sHelpText ;			// help on this command

public:
	CPsyMudCommand() 	{}
	// szCommandText is actual command, szCategory is the classification for the command
	// like social, battle, informational etc.  szHelpText is what is displayed when
	// the user asks for help on this command
	CPsyMudCommand(char * szCommandText, char * szCategory, char * szHelpText) ;
	virtual ~CPsyMudCommand() ;

	virtual void ProcessCommand(const string& sRestOfCommandLine, CPsyMudPCharacter * pch) = 0 ;
	virtual void ShowHelp(CPsyMudPCharacter * pch) ;
	inline const char * GetCommandText()	{ return m_sCommandText.c_str() ; }

	// this is used for things like builder menus.  The command
	// wants all the user's input until they exit the menu
	virtual bool StillWantsAllInput() 		
		{ return false ; }

	// this is called for all the additional input.  ProcessCommand
	// is called only for the initial input.
	virtual void ProcessAdditionalInput(char * szInput, CPsyMudPCharacter * pch)	
		{}

	bool operator==(const CPsyMudCommand& rhs)
		{	
			// on this one we're actually comparing objects
			// so we require the whole string to match
			return m_sCommandText == rhs.m_sCommandText ;
		}

	bool operator==(const char * rhs)
		{
			// this is so we can find the command that
			// starts with rhs.  A complete match is
			// not necessary
			return strncasecmp(m_sCommandText.c_str(), rhs, strlen(rhs)) == 0 ;
		}
		
	
} ;

#endif	//#define	CPSYMUDCOMMAND_H
