//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================



#include <algorithm>
#include "CPsyMudCommandProcessor.h"

// begin command includes
#include "CPsyMudCommand.h"		// base object

#include "CPsyMudCmdAlias.h"		// alias
#include "CPsyMudCmdDrop.h"			// drop
#include "CPsyMudCmdGet.h"			// get
#include "CPsyMudCmdGo.h"			// go
#include "CPsyMudCmdGoto.h"			// goto
#include "CPsyMudCmdInventory.h"	// inventory
#include "CPsyMudCmdLook.h"			// look
#include "CPsyMudCmdQuit.h"			// quit
#include "CPsyMudCmdRest.h"			// rest
#include "CPsyMudCmdSay.h"			// say
#include "CPsyMudCmdScore.h"		// score
#include "CPsyMudCmdSit.h"			// sit
#include "CPsyMudCmdSleep.h"		// sleep
#include "CPsyMudCmdStand.h"		// stand
#include "CPsyMudCmdWake.h"			// wake
#include "CPsyMudCmdWho.h"			// who

// end command includes

#define INSERT_CMD(x)	m_Commands.insert(m_Commands.end(), x)
//#define ADD_CMD(x)		cmd = new x ; m_Commands.insert(m_Commands.end(), x)

CPsyMudCommandProcessor	g_CommandProcessor ;


CPsyMudCommandProcessor::CPsyMudCommandProcessor()
{
	CPsyMudCommand * cmd ;

	// here is where you add new commands.  Pre is that you
	// actually built a class module to represent the command
	// and you included its header file at the top here
	//
	// order here is important.  Commands are found by their
	// order in this list.  As you know, a user can type just
	// a few characters of a command.  The command is then
	// searched (linearly) in this list.  First hit gets it
	//
	// NOTE: To remove any commands you don't want in your
	// MUD, simply comment out the line here.  If you want to
	// make the binary smaller (not by much), comment out the
	// command here, comment out the include at the top and
	// remove the command from the makefile.
	//
	cmd = new CPsyMudCmdAlias ;		INSERT_CMD(cmd) ;	// alias	- add / view / remove aliases
//	cmd = new CPsyMudCmdApplaud ;	INSERT_CMD(cmd) ;	// applaud	- social
//	cmd = new CPsyMudCmdAssist ;	INSERT_CMD(cmd) ;	// assist	- help pc/npc with fights

//	cmd = new CPsyMudCmdBleed ;		INSERT_CMD(cmd) ;	// bleed	- social
//	cmd = new CPsyMudCmdBlush ;		INSERT_CMD(cmd) ;	// blush	- social
//	cmd = new CPsyMudCmdBounce ;	INSERT_CMD(cmd) ;	// bounce	- social
//	cmd = new CPsyMudCmdBow ;		INSERT_CMD(cmd) ;	// bow		- social
//	cmd = new CPsyMudCmdBurp ;		INSERT_CMD(cmd) ;	// burp		- social

//	cmd = new CPsyMudCmdCackle ;	INSERT_CMD(cmd) ;	// cackle	- social
//	cmd = new CPsyMudCmdCast ;		INSERT_CMD(cmd) ;	// cast		- cast spell
//	cmd = new CPsyMudCmdChuckle ;	INSERT_CMD(cmd) ;	// chuckle	- social
//	cmd = new CPsyMudCmdClap ;		INSERT_CMD(cmd) ;	// clap		- social
//	cmd = new CPsyMudCmdClose ;		INSERT_CMD(cmd) ;	// close	- close something that was previously opened
//	cmd = new CPsyMudCmdComb ;		INSERT_CMD(cmd) ;	// comb		- social
//	cmd = new CPsyMudCmdComfort ;	INSERT_CMD(cmd) ;	// comfort	- social
//	cmd = new CPsyMudCmdCommands ;	INSERT_CMD(cmd) ;	// commands	- lists all available commands
//	cmd = new CPsyMudCmdConsider ;	INSERT_CMD(cmd) ;	// consider	- consider attacking an npc/pc.  displays difficulty
//	cmd = new CPsyMudCmdCough ;		INSERT_CMD(cmd) ;	// cough	- social
//	cmd = new CPsyMudCmdCringe ;	INSERT_CMD(cmd) ;	// cringe	- social
//	cmd = new CPsyMudCmdCry ;		INSERT_CMD(cmd) ;	// cry		- social
//	cmd = new CPsyMudCmdCuddle ;	INSERT_CMD(cmd) ;	// cuddle	- social
//	cmd = new CPsyMudCmdCurse ;		INSERT_CMD(cmd) ;	// curse	- social
//	cmd = new CPsyMudCmdCurtsey ;	INSERT_CMD(cmd) ;	// curtsey	- social

//	cmd = new CPsyMudCmdDance ;		INSERT_CMD(cmd) ;	// dance	- social
//	cmd = new CPsyMudCmdDaydream ;	INSERT_CMD(cmd) ;	// daydream	- social
//	cmd = new CPsyMudCmdDonate ;	INSERT_CMD(cmd) ;	// donate	- donate an item
//	cmd = new CPsyMudCmdDrink ;		INSERT_CMD(cmd) ;	// drink	- drink liquid
//	cmd = new CPsyMudCmdDrool ;		INSERT_CMD(cmd) ;	// drool	- social
	cmd = new CPsyMudCmdDrop ;		INSERT_CMD(cmd) ;	// drop		- drop item

//	cmd = new CPsyMudCmdEat ;		INSERT_CMD(cmd) ;	// eat		- eat food
//	cmd = new CPsyMudCmdEmbrace ;	INSERT_CMD(cmd) ;	// embrace	- social
//	cmd = new CPsyMudCmdEmote ;		INSERT_CMD(cmd) ;	// emote	- generic social.  can emote anything
//	cmd = new CPsyMudCmdEmpty ;		INSERT_CMD(cmd) ;	// empty	- empty liquid from container
//	cmd = new CPsyMudCmdEquipment ;	INSERT_CMD(cmd) ;	// equipment- show equipment

//	cmd = new CPsyMudCmdFart ;		INSERT_CMD(cmd) ;	// fart		- social
//	cmd = new CPsyMudCmdFill ;		INSERT_CMD(cmd) ;	// fill 	- fill container with liquid
//	cmd = new CPsyMudCmdFlee ;		INSERT_CMD(cmd) ;	// flee		- flee a fight
//	cmd = new CPsyMudCmdFlip ;		INSERT_CMD(cmd) ;	// flip		- social
//	cmd = new CPsyMudCmdFlirt ;		INSERT_CMD(cmd) ;	// flirt	- social
//	cmd = new CPsyMudCmdFollow ;	INSERT_CMD(cmd) ;	// follow	- follow a pc/npc
//	cmd = new CPsyMudCmdFondle ;	INSERT_CMD(cmd) ;	// fondle	- social
//	cmd = new CPsyMudCmdFrench ;	INSERT_CMD(cmd) ;	// french	- social
//	cmd = new CPsyMudCmdFrown ;		INSERT_CMD(cmd) ;	// frown	- social
//	cmd = new CPsyMudCmdFume ;		INSERT_CMD(cmd) ;	// fume		- social

//	cmd = new CPsyMudCmdGasp ;		INSERT_CMD(cmd) ;	// gasp		- social
	cmd = new CPsyMudCmdGet ;		INSERT_CMD(cmd) ;	// get		- get something from ground
//	cmd = new CPsyMudCmdGiggle ;	INSERT_CMD(cmd) ;	// giggle	- social
//	cmd = new CPsyMudCmdGive ;		INSERT_CMD(cmd) ;	// give		- give something to someone
//	cmd = new CPsyMudCmdGlare ;		INSERT_CMD(cmd) ;	// glare	- social
	cmd = new CPsyMudCmdGo ;		INSERT_CMD(cmd) ;	// go 		- alias for n, s, e, w, ne, nw, se, sw added by new char code
//	cmd = new CPsyMudCmdGold ;		INSERT_CMD(cmd) ;	// gold		- show how much gold I have
	cmd = new CPsyMudCmdGoto ;		INSERT_CMD(cmd) ;	// goto		- immortal command to go to any room
//	cmd = new CPsyMudCmdGossip ;	INSERT_CMD(cmd) ;	// gossip	- gossip on the gossip channel
//	cmd = new CPsyMudCmdGrats ;		INSERT_CMD(cmd) ;	// grats	- say something on the grats channel
//	cmd = new CPsyMudCmdGreet ;		INSERT_CMD(cmd) ;	// greet	- social
//	cmd = new CPsyMudCmdGrin ;		INSERT_CMD(cmd) ;	// grin		- social
//	cmd = new CPsyMudCmdGroan ;		INSERT_CMD(cmd) ;	// groan	- social
//	cmd = new CPsyMudCmdGrope ;		INSERT_CMD(cmd) ;	// grope	- social
//	cmd = new CPsyMudCmdGroup ;		INSERT_CMD(cmd) ;	// group	- group with pc/npc
//	cmd = new CPsyMudCmdGrovel ;	INSERT_CMD(cmd) ;	// grovel	- social
//	cmd = new CPsyMudCmdGrowl ;		INSERT_CMD(cmd) ;	// growl	- social
//	cmd = new CPsyMudCmdGsay ;		INSERT_CMD(cmd) ;	// gsay		- say something to every member of your group

//	cmd = new CPsyMudCmdHelp ;		INSERT_CMD(cmd) ;	// help		- help on a command
//	cmd = new CPsyMudCmdHiccup ;	INSERT_CMD(cmd) ;	// hiccup	- social
//	cmd = new CPsyMudCmdHide ;		INSERT_CMD(cmd) ;	// hide		- hide in shadows
//	cmd = new CPsyMudCmdHold ;		INSERT_CMD(cmd) ;	// hold		- hold item
//	cmd = new CPsyMudCmdHoller ;	INSERT_CMD(cmd) ;	// holler	- social
//	cmd = new CPsyMudCmdHop ;		INSERT_CMD(cmd) ;	// hop		- social
//	cmd = new CPsyMudCmdHug ;		INSERT_CMD(cmd) ;	// hug		- social

	cmd = new CPsyMudCmdInventory ;	INSERT_CMD(cmd) ;	// inventory- look in your inventory
//	cmd = new CPsyMudCmdInsult ;	INSERT_CMD(cmd) ;	// insult	- social

//	cmd = new CPsyMudCmdJunk ;		INSERT_CMD(cmd) ;	// junk		- junk an object (destroys that instance)

//	cmd = new CPsyMudCmdKill ;		INSERT_CMD(cmd) ;	// kill		- kill pc/npc
//	cmd = new CPsyMudCmdKiss ;		INSERT_CMD(cmd) ;	// kiss		- social

//	cmd = new CPsyMudCmdLag ;		INSERT_CMD(cmd) ;	// lag		- social
	cmd = new CPsyMudCmdLook ;		INSERT_CMD(cmd) ;	// look		- look at room or something
//	cmd = new CPsyMudCmdLaugh ;		INSERT_CMD(cmd) ;	// laugh	- social
//	cmd = new CPsyMudCmdLevels ;	INSERT_CMD(cmd) ;	// levels	- display level information
//	cmd = new CPsyMudCmdLoad ;		INSERT_CMD(cmd) ;	// load 	- load an instance of mud object/npc etc into existance
//	cmd = new CPsyMudCmdLock ;		INSERT_CMD(cmd) ;	// lock		- lock something
//	cmd = new CPsyMudCmdLove ;		INSERT_CMD(cmd) ;	// love		- social

//	cmd = new CPsyMudCmdMail ;		INSERT_CMD(cmd) ;	// mail		- invoke the mail system
//	cmd = new CPsyMudCmdMassage ;	INSERT_CMD(cmd) ;	// massage	- social
//	cmd = new CPsyMudCmdMoan ;		INSERT_CMD(cmd) ;	// moan		- social
//	cmd = new CPsyMudCmdMute ;		INSERT_CMD(cmd) ;	// mute		- mute an annoyint player - IMMs only

//	cmd = new CPsyMudCmdNewbie ;	INSERT_CMD(cmd) ;	// newbie	- say something on the newbie channel
//	cmd = new CPsyMudCmdNibble ;	INSERT_CMD(cmd) ;	// nibble	- social
//	cmd = new CPsyMudCmdNod ;		INSERT_CMD(cmd) ;	// nod		- social
//	cmd = new CPsyMudCmdNudge ;		INSERT_CMD(cmd) ;	// nudge	- social
//	cmd = new CPsyMudCmdNuzzle ;	INSERT_CMD(cmd) ;	// nuzzle	- social

//	cmd = new CPsyMudCmdOLC ;		INSERT_CMD(cmd) ;	// olc 		- online mud editor
//	cmd = new CPsyMudCmdOpen ;		INSERT_CMD(cmd) ;	// open		- open something (door etc)

//	cmd = new CPsyMudCmdPat ;		INSERT_CMD(cmd) ;	// pat		- social
//	cmd = new CPsyMudCmdPoint ;		INSERT_CMD(cmd) ;	// point	- social
//	cmd = new CPsyMudCmdPoke ;		INSERT_CMD(cmd) ;	// poke		- social
//	cmd = new CPsyMudCmdPonder ;	INSERT_CMD(cmd) ;	// ponder	- social
//	cmd = new CPsyMudCmdPoop ;		INSERT_CMD(cmd) ;	// poop		- social
//	cmd = new CPsyMudCmdPout ;		INSERT_CMD(cmd) ;	// pout		- social
//	cmd = new CPsyMudCmdPractice ;	INSERT_CMD(cmd) ;	// practice	- practice spell / skill
//	cmd = new CPsyMudCmdPray ;		INSERT_CMD(cmd) ;	// pray		- social
//	cmd = new CPsyMudCmdPuke ;		INSERT_CMD(cmd) ;	// puke		- social
//	cmd = new CPsyMudCmdPurr ;		INSERT_CMD(cmd) ;	// purr		- social
//	cmd = new CPsyMudCmdPut ;		INSERT_CMD(cmd) ;	// put		- put item in container

//	cmd = new CPsyMudCmdQuaff ;		INSERT_CMD(cmd) ;	// quaff	- quaff potion
	cmd = new CPsyMudCmdQuit ;		INSERT_CMD(cmd) ;	// quit		- quit the game

//	cmd = new CPsyMudCmdRead ;		INSERT_CMD(cmd) ;	// read		- read book
//	cmd = new CPsyMudCmdRecite ;	INSERT_CMD(cmd) ;	// recite	- recite scroll
//	cmd = new CPsyMudCmdReply ;		INSERT_CMD(cmd) ;	// reply	- reply to last tell
	cmd = new CPsyMudCmdRest ;		INSERT_CMD(cmd) ;	// rest		- rest
//	cmd = new CPsyMudCmdRuffle ;	INSERT_CMD(cmd) ;	// ruffle	- social

	cmd = new CPsyMudCmdSay ;		INSERT_CMD(cmd) ;	// say		- say something to people in the room
//	cmd = new CPsyMudCmdSave ;		INSERT_CMD(cmd) ;	// save		- save character information
	cmd = new CPsyMudCmdScore ;		INSERT_CMD(cmd) ;	// score 	- view my score info
//	cmd = new CPsyMudCmdScream ;	INSERT_CMD(cmd) ;	// scream	- social
//	cmd = new CPsyMudCmdSet ;		INSERT_CMD(cmd) ;	// set		- set an option (includes color, title etc.)
//	cmd = new CPsyMudCmdShake ;		INSERT_CMD(cmd) ;	// shake	- social
//	cmd = new CPsyMudCmdShiver ;	INSERT_CMD(cmd) ;	// shiver	- social
//	cmd = new CPsyMudCmdShout ;		INSERT_CMD(cmd) ;	// shout	- shout something to people in your world
//	cmd = new CPsyMudCmdShow ;		INSERT_CMD(cmd) ;	// show		- stop hiding
//	cmd = new CPsyMudCmdShrug ;		INSERT_CMD(cmd) ;	// shrug	- social
//	cmd = new CPsyMudCmdSigh ;		INSERT_CMD(cmd) ;	// sigh		- social
//	cmd = new CPsyMudCmdSing ;		INSERT_CMD(cmd) ;	// sing		- social
	cmd = new CPsyMudCmdSit ;		INSERT_CMD(cmd) ;	// sit		- sit down
//	cmd = new CPsyMudCmdSlap ;		INSERT_CMD(cmd) ;	// slap		- social
	cmd = new CPsyMudCmdSleep ;		INSERT_CMD(cmd) ;	// sleep	- go to sleep
//	cmd = new CPsyMudCmdSmile ;		INSERT_CMD(cmd) ;	// smile	- social
//	cmd = new CPsyMudCmdSmirk ;		INSERT_CMD(cmd) ;	// smirk	- social
//	cmd = new CPsyMudCmdSnap ;		INSERT_CMD(cmd) ;	// snap		- social
//	cmd = new CPsyMudCmdSnarl ;		INSERT_CMD(cmd) ;	// snarl	- social
//	cmd = new CPsyMudCmdSneeze ;	INSERT_CMD(cmd) ;	// sneeze	- social
//	cmd = new CPsyMudCmdSniff ;		INSERT_CMD(cmd) ;	// sniff	- social
//	cmd = new CPsyMudCmdSnicker ;	INSERT_CMD(cmd) ;	// snicker	- social
//	cmd = new CPsyMudCmdSniff ;		INSERT_CMD(cmd) ;	// sniff	- social
//	cmd = new CPsyMudCmdSnore ;		INSERT_CMD(cmd) ;	// snore	- social
//	cmd = new CPsyMudCmdSnuggle ;	INSERT_CMD(cmd) ;	// snuggle	- social
//	cmd = new CPsyMudCmdSpank ;		INSERT_CMD(cmd) ;	// spank	- social
//	cmd = new CPsyMudCmdSpit ;		INSERT_CMD(cmd) ;	// spit		- social
//	cmd = new CPsyMudCmdSqueeze ;	INSERT_CMD(cmd) ;	// squeeze	- social
	cmd = new CPsyMudCmdStand ;		INSERT_CMD(cmd) ;	// stand	- stand
//	cmd = new CPsyMudCmdStare ;		INSERT_CMD(cmd) ;	// stare	- social
//	cmd = new CPsyMudCmdStroke ;	INSERT_CMD(cmd) ;	// stroke	- social
//	cmd = new CPsyMudCmdStrut ;		INSERT_CMD(cmd) ;	// strut	- social
//	cmd = new CPsyMudCmdSulk ;		INSERT_CMD(cmd) ;	// sulk		- social
//	cmd = new CPsyMudCmdSwallow ;	INSERT_CMD(cmd) ;	// swallow	- swallow pill

//	cmd = new CPsyMudCmdTake ;		INSERT_CMD(cmd) ;	// take		- take something from container
//	cmd = new CPsyMudCmdTango ;		INSERT_CMD(cmd) ;	// tango	- social
//	cmd = new CPsyMudCmdTaunt ;		INSERT_CMD(cmd) ;	// taunt	- social
//	cmd = new CPsyMudCmdTell ;		INSERT_CMD(cmd) ;	// tell		- tell someone something
//	cmd = new CPsyMudCmdThank ;		INSERT_CMD(cmd) ;	// thank	- social
//	cmd = new CPsyMudCmdThink ;		INSERT_CMD(cmd) ;	// think	- social
//	cmd = new CPsyMudCmdTickle ;	INSERT_CMD(cmd) ;	// tickle	- social
//	cmd = new CPsyMudCmdTrain ;		INSERT_CMD(cmd) ;	// train	- train attribute
//	cmd = new CPsyMudCmdTurn ;		INSERT_CMD(cmd) ;	// turn		- turn something
//	cmd = new CPsyMudCmdTwiddle ;	INSERT_CMD(cmd) ;	// twiddle	- social

//	cmd = new CPsyMudCmdUngroup ;	INSERT_CMD(cmd) ;	// ungroup	- get out of a group
//	cmd = new CPsyMudCmdUnlock ;	INSERT_CMD(cmd) ;	// unlock	- unlock something
//	cmd = new CPsyMudCmdUse ;		INSERT_CMD(cmd) ;	// use		- use something

//	cmd = new CPsyMudCmdVersion ;	INSERT_CMD(cmd) ;	// version	- returns version and copyright information

	cmd = new CPsyMudCmdWake ;		INSERT_CMD(cmd) ;	// wake		- wake up
//	cmd = new CPsyMudCmdWave ;		INSERT_CMD(cmd) ;	// wave		- social
//	cmd = new CPsyMudCmdWear ;		INSERT_CMD(cmd) ;	// wear		- wear equipment
//	cmd = new CPsyMudCmdWhine ;		INSERT_CMD(cmd) ;	// whine	- social
//	cmd = new CPsyMudCmdWhistle ;	INSERT_CMD(cmd) ;	// whistle	- social
//	cmd = new CPsyMudCmdWield ;		INSERT_CMD(cmd) ;	// wield	- wield weapon
//	cmd = new CPsyMudCmdWiggle ;	INSERT_CMD(cmd) ;	// wiggle	- social
//	cmd = new CPsyMudCmdWimpy ;		INSERT_CMD(cmd) ;	// wimpy	- set hp under which you autoflee
//	cmd = new CPsyMudCmdWink ;		INSERT_CMD(cmd) ;	// wink		- social
	cmd = new CPsyMudCmdWho ;		INSERT_CMD(cmd) ;	// who		- who is playing
//	cmd = new CPsyMudCmdWhoami ;	INSERT_CMD(cmd) ;	// whoami	- gives information about me
//	cmd = new CPsyMudCmdWhois ;		INSERT_CMD(cmd) ;	// whois	- whois a particular player

//	cmd = new CPsyMudCmdYawn ;		INSERT_CMD(cmd) ;	// yawn		- social
//	cmd = new CPsyMudCmdYodel ;		INSERT_CMD(cmd) ;	// yodel	- social

//	cmd = new CPsyMudCmdZit ;		INSERT_CMD(cmd) ;	// zit		- social (pop a zit)
	
}

CPsyMudCommandProcessor::~CPsyMudCommandProcessor()
{
	// TO DO: delete all objects in vector
}


CPsyMudCommand * CPsyMudCommandProcessor::FindCommand(const char * szCommand) const
{
	for (vector<CPsyMudCommand *>::const_iterator p = m_Commands.begin() ; p < m_Commands.end() ; p++)
	{
		if (*(*p) == szCommand)
		{
			return (*p) ;
		}
	}
	
	// no matching command
	return NULL ;

}