//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================



#ifndef CPSYMUDITEM_H
#define CPSYMUDITEM_H

#include <string>
#include "CPsyMudObject.h"

const unsigned short int	MAX_ITEM_SIZE = 10000 ;
const unsigned short int	MAX_ITEM_WEIGHT = 10000 ;
const unsigned short int	MAX_ITEM_NAMES = 5 ;


class CPsyMudItem : public CPsyMudObject
{
public:
	enum EPsyMudItemType
	{
		itmMisc,
		itmArmor,
		itmWeapon,
		itmPill,
		itmPotion,
		itmWand,
		itmBook,
		itmFood,
		itmDrink
	} ;

	enum EPsyMudItemWearLocation
	{
		itmWieldSingle 		= 1 << 0,
		itmWearHand 		= 1 << 1,
		itmWearShield 		= 1 << 2,
		itmWearHead 		= 1 << 3,
		itmWearEye 			= 1 << 4,
		itmWearEar 			= 1 << 5,
		itmWearFace 		= 1 << 6,
		itmWearNose 		= 1 << 7,
		itmWearMouth 		= 1 << 8,
		itmWearNeck 		= 1 << 9,
		itmWearOverBody 	= 1 << 10,
		itmWearShoulder 	= 1 << 11,
		itmWearTorso 		= 1 << 12,
		itmWearWaist 		= 1 << 13,
		itmWearCrotch 		= 1 << 14,
		itmWearWrist 		= 1 << 15,
		itmWearLegs 		= 1 << 16,
		itmWearFeet 		= 1 << 17,
		itmWearRingFinger 	= 1 << 18,
		itmWearRingOther 	= 1 << 19,
		itmWearBadge 		= 1 << 20,
		itmwearHold			= 1 << 21,				// if you get armor etc bennies by holding

		itmSticky			= 1 << 30,				// set this only if item is !drop
		itmWearTwoHandedModifier 	= 1 << 31		// set if this is a two handed hold/wield
	} ;

	enum EPsyMudItemAttributes
	{
		itmAttribGlow	= 1 << 0,
		itmAttribHum	= 1 << 1,
	} ;

protected:
	unsigned short int	m_iWeight ;				// weight of the item
	unsigned short int	m_iSize ;				// size of the item, 0 to MAX_ITEM_SIZE.  1 would be like a fine grain of sand and
												// MAX_ITEM_SIZE would be something like a planet

	string	m_sLongDescription ;				// long description of this object (usually a paragraph)
	string	m_sShortDescription	;				// short description of this object (a single line, used as the "name")
	vector<string> m_Aliases ;				// list of object names/aliases


	EPsyMudItemType			m_type ;	
	EPsyMudItemWearLocation	m_wear ;
	EPsyMudItemAttributes 	m_attrib ;

	//CPsyMudScript * m_script	

public:
	virtual ~CPsyMudItem()	{}

	inline const string		GetShortDescription() const				{ return m_sShortDescription ; }
	inline void				SetShortDescription(const string& s)	{ m_sShortDescription = s ; }

	inline const string		GetLongDescription() const				{ return m_sLongDescription ; }
	inline void				SetLongDescription(const string& s)		{ m_sLongDescription = s ; }

	inline void				AddAlias(const string& sAlias)			{ m_Aliases.insert(m_Aliases.end(), sAlias) ; }
	const bool				HasAlias(const string& sAlias) const ;


	void SerializeIn(void * data) 		{}
	void SerializeOut(void * data)		{}

} ;


#endif //#define CPSYMUDITEM_H
