//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================



#include <iostream>
//#include <stdio.h>
#include <iomanip.h>
#include <stdarg.h>
#include <sys/time.h>
#include "CPsyMudLog.h"

CPsyMudLog g_Log ;


string * CPsyMudLog::GetTimeStamp()
{
	time_t tv_time = time(0) ;
	tm t = *localtime(&tv_time) ;
	char szCurrentTime[MUD_LOG_TIMESTAMP_SIZE+1] ;

	string * sTimeStamp ;

	strftime(szCurrentTime, MUD_LOG_TIMESTAMP_SIZE, "%b %d %H:%M:%S", &t) ;

	sTimeStamp = new string(szCurrentTime) ;

	//sTimeStamp = (char *)szCurrentTime ;

	return sTimeStamp ;
}


CPsyMudLog::CPsyMudLog(char * szLogFileName = MUDLOGFILENAME) throw(CPsyMudLogException)
{
	// open log file for output and append
	m_LogFile.open(szLogFileName, ios::out | ios::app) ;

	// throw an exception if log file failed to open.  Messages are going to
	// stderr at this point simply because the log is not yet open.
	if (!m_LogFile)
	{
		cerr << "System Error : Log file \"" << szLogFileName << "\" failed to open for write." << endl ;
		throw CPsyMudLogException("Unable to open log file for output.") ;
	}
	else
	{
		cerr << "System Info : Log file \"" << szLogFileName << "\" opened successfully." << endl ;
		Log(logGeneral, "Log initialized with file \"%s\". ", szLogFileName) ;
	}
}

CPsyMudLog::~CPsyMudLog()
{
	if (!m_LogFile)
	{
		// should never get here
		cerr << "System Error : CPsyMudLog::~CPsyMudLog Log file not opened.  Something funky going on here." << endl ;
	}
	else
	{
		// log the close and then actually close the log
		Log(logGeneral, "Log terminated.") ;	// implicit flush as nice side effect
		m_LogFile.close() ;

// Commented out.  Doesn't work as expected.		
//		if (!m_LogFile)
//		{
//			cerr << "System Info : CPsyMudLog::~CPsyMudLog Log closed." << endl ;
//		}
//		else
//		{
//			cerr << "System Error : CPsyMudLog::~CPsyMudLog Log failed to close properly." << endl ;
//		}
	}
}



void CPsyMudLog::Log(LogType typ, char * pszFormat, ...) throw(CPsyMudLogException)
{
	va_list args ;
	string sMessage ;	

	// thought about making this throw an exception.  Probably not
	// worth bringing down the mud over a bad log message, though :-)
	if (pszFormat == NULL)
	{
		pszFormat = "System Error : CPsyMudLog::Log() received a NULL format." ;
	}


	va_start(args, pszFormat) ;
	vsnprintf(m_szMessageWorkArea, MAX_LOG_MESSAGE_SIZE, pszFormat, args) ;
	va_end(args) ;

	// TEMP!!!
	cerr << "DEBUG: Log: " << m_szMessageWorkArea << endl ;
	// end temp
	
	m_LogFile << GetTimeStamp()->c_str() << " "  ;

	switch (typ)
	{
		case logTrace :
			m_LogFile << "trace    " ;
			break ;
		case logGeneral :
			m_LogFile << "general  " ;
			break ;
		case logError :
			m_LogFile << "* error *" ;
			break ;
		case logSecurity :
			m_LogFile << "security " ;
			break ;
		case logSpecialCommands :
			m_LogFile << "special  " ;
			break ;
		case logRestart :
			m_LogFile << "restart  " ;
			break ;
		case logNewPlayer :
			m_LogFile << "newplayer" ;
			break ;		
		default :
			m_LogFile << " **???** " ;
			break ;		
	}

	m_LogFile << " : " << m_szMessageWorkArea << endl << flush ;

	return ;
}