//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================


#include "CPsyMudPCharacter.h"
#include <string>
#include <stdio.h>	// for sprintf

//#define MAX_PROMPT_SIZE 50

const string CPsyMudPCharacter::BuildPrompt() const	
{
	static char szHP[8] ;
	static char szMV[8] ;
	static char szMA[8] ;

	string sPrompt ;		// we build the prompt into this

	sprintf(szHP, "%d", GetPoints()->GetCurrentHitPoints()) ;
	sprintf(szMA, "%d", GetPoints()->GetCurrentMana()) ;
	sprintf(szMV, "%d", GetPoints()->GetCurrentMovementPoints()) ;

	// TO DO: Make red and blinking hp if below wimpy amount, make
	// green if poisoned etc.  Maybe also add current position to prompt
	// (sleeping, standing, sitting etc - some abbreviation for it)
	
	sPrompt = "" ;
	sPrompt += COLOR_BRIGHT_GREEN + string(szHP) + COLOR_DARK_YELLOW + string("hp ") ;
	sPrompt += COLOR_BRIGHT_GREEN + string(szMA) + COLOR_DARK_YELLOW + string("ma ") ;
	sPrompt += COLOR_BRIGHT_GREEN + string(szMV) + COLOR_DARK_YELLOW + string("mv ") ;

	// TO DO: Add exp required for next level

	sPrompt += COLOR_DARK_RED+ string("> ") + COLOR_NORMAL ;

	return sPrompt ;
}

void CPsyMudPCharacter::AddAlias(const string sAlias, const string sDefinition)
{
	m_Aliases.insert(m_Aliases.end(), CPsyMudCommandAlias(sAlias, sDefinition)) ;
}

const string CPsyMudPCharacter::GetAliasDefinition (const string sAlias) const
{
	for (vector<CPsyMudCommandAlias>::const_iterator p = m_Aliases.begin() ; p < m_Aliases.end() ; p++)
	{
		if (p->GetAlias() == sAlias)
		{
			return p->GetDefinition() ;
		}
	}

	return string("") ;
}

bool CPsyMudPCharacter::RemoveAlias (const string sAlias)
{
	for (vector<CPsyMudCommandAlias>::const_iterator p = m_Aliases.begin() ; p < m_Aliases.end() ; p++)
	{
		if (p->GetAlias() == sAlias)
		{
			m_Aliases.erase(p) ;
			return true ;
		}
	}

	return false ;
}

