//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================



#ifndef CPSYMUDPCHARACTER_H
#define CPSYMUDPCHARACTER_H

#include "CPsyMudComm.h"
#include "CPsyMudCharacter.h"
#include "color.h"

#define PCHAR_STATE_UNKNOWN					0
#define PCHAR_STATE_NEEDS_LOGIN_PROMPT		1
#define PCHAR_STATE_NEEDS_PASSWORD_PROMPT	2

// ===============================================================================

class CPsyMudCommandAlias
{
private:
	string 	m_sAlias ;
	string	m_sExpandsTo ;

public:
	CPsyMudCommandAlias()	{}
	CPsyMudCommandAlias(const string& sAlias, const string& sDefinition)
	{
		m_sAlias = sAlias ;
		m_sExpandsTo = sDefinition ;
	}

	const string	GetAlias() const		{ return m_sAlias ; }
	const string	GetDefinition() const	{ return m_sExpandsTo ; }

} ;

// ===============================================================================

class CPsyMudPCharacter : public CPsyMudCharacter
{
private:
	bool				m_bVerboseMode ;	// true if they autolook in a room when they enter the room
	bool 				m_bAutoListExits ;	// true if when "look"ing, we list exits in the room as well

	unsigned long		m_ulState ;			// needs logon, needs password, etc.

	CPsyMudConnInfo *	m_conn ;			// pointer to connection object.  Don't delete in destructor or = and
											// copy constructor will bomb with lists/map/vector etc.
	vector <CPsyMudCommandAlias> m_Aliases ;	

public:
	CPsyMudPCharacter() 									
		{
			m_bVerboseMode = true ;
			m_bAutoListExits = true ;
		}
	CPsyMudPCharacter(const CPsyMudPCharacter& copy) : CPsyMudCharacter(copy)		
		{
			m_conn = copy.m_conn ;
			m_bVerboseMode = copy.m_bVerboseMode ;
			m_bAutoListExits = copy.m_bAutoListExits ;
		}
	CPsyMudPCharacter(CPsyMudConnInfo * inf)				
		{
			m_bVerboseMode = true ;
			m_bAutoListExits = true ;
			m_conn = inf ;
		}

	~CPsyMudPCharacter()									{ /* delete CPsyMudConnectionInfo ; */ }

	void operator=  (const CPsyMudPCharacter& rhs) 			{ (CPsyMudCharacter)*this = (CPsyMudCharacter)rhs ; }
	bool operator== (const CPsyMudPCharacter& rhs) const 	{ return GetObjectID() == rhs.GetObjectID() ; }
	bool operator<  (const CPsyMudPCharacter& rhs) const	{ return GetObjectID() < rhs.GetObjectID() ; } // TO DO: Check this

	void SerializeIn(void * data) 							{}
	void SerializeOut(void * data) 							{}

	const vector <CPsyMudCommandAlias>* GetAliases() const 	{ return &m_Aliases ; }

	void			AddAlias(const string sAlias, const string sDefinition) ;
	const string	GetAliasDefinition (const string sAlias) const ;
	bool			RemoveAlias (const string sAlias) ;
	

	inline CPsyMudConnInfo * GetConnectionInfo()			{ return m_conn ; }
	inline void SetConnectionInfo(CPsyMudConnInfo * nc)		{ m_conn = nc ; }
	
	const string BuildPrompt() const ;

	unsigned long	GetState() const						{ return m_ulState ; }
	inline void		SetState(const unsigned long l)			{ m_ulState = l; }

	inline bool		GetVerboseMode()						{ return m_bVerboseMode ; }
	inline void		SetVerboseMode(const bool b)			{ m_bVerboseMode = b ; }

	inline bool		GetAutoExits()							{ return m_bAutoListExits ; }
	inline void		SetAutoExits(const bool b)				{ m_bAutoListExits = b ; }


} ;

#endif //  CPSYMUDPCHARACTER_H