//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================

#include <vector>
#include <stdlib.h>
#include "CPsyMudRoom.h"
#include "CPsyMudRoomExit.h"

bool CPsyMudRoom::AddExit(CPsyMudRoomExit * ExitToAdd)
{
	// first make sure the name doesn't conflict with another exit in the room
	for (vector<CPsyMudRoomExit *>::const_iterator p = m_Exits.begin() ; p < m_Exits.end() ; p++)
	{
		if ((*p)->GetDirectionName() == ExitToAdd->GetDirectionName())
		{
			// found exit with same name
			return false ;
		}
	}

	// add the exit to the list
	m_Exits.insert(m_Exits.end(), ExitToAdd) ;
	return true ;
}

// must provide full exit name when removing by name
bool CPsyMudRoom::RemoveExitByName(const char * szName)
{
	// find the exit
	for (vector<CPsyMudRoomExit *>::const_iterator p = m_Exits.begin() ; p < m_Exits.end() ; p++)
	{
		if (strcasecmp((*p)->GetDirectionName().c_str(), szName) == 0)
		{
			// found exit with same name
			m_Exits.erase(p) ;
			return true ;
		}
	}
	return false ;
}

CPsyMudRoomExit * CPsyMudRoom::GetExitByName(const char * szName) const
{
	// find the exit
	for (vector<CPsyMudRoomExit *>::const_iterator p = m_Exits.begin() ; p < m_Exits.end() ; p++)
	{
		// changed because if northeast (example) was in the list before "north", you
		// could never go north.
		//if (strncasecmp((*p)->GetDirectionName().c_str(), szName, strlen(szName)) == 0)
		if (strcasecmp((*p)->GetDirectionName().c_str(), szName) == 0)
		{
			return (*p) ;
		}
	}

	return NULL ;
}
