//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================

#ifndef CPSYMUDROOMEXIT_H
#define CPSYMUDROOMEXIT_H

#include <string>

#include "CPsyMudObject.h"
//#include "CPsyMudRoom.h"

// an exit is a one-way link to another room.  To link two rooms, in a bidrectional manner you need two exits
// one going from room a to room b, and the other going from room b to room a.  This may seem like more work,
// but it allows for maximum flexibility in designing the areas

// TO DO: Need to do checking in Set method based upon this length
#define MAX_EXIT_DIRECTION_NAME_LEN	50

class CPsyMudRoomExit
{
private:
	string 	m_sDirectionName ;		// not limited to north, south, east, west etc.  can also be things like "behind", "underneath" etc.
	string	m_sDescription ;		// description of the exit.  defaults to linked room name

	//CPsyMudRoom * m_Destination ;	// room this exit goes to
	CPsyMudObject * m_Destination ;	// room this exit goes to.  Had to declare as object because of circular
									// dependency.  Must cast to CPsyMudRoom in actual code

	bool	m_bDefaultLocked ;		// true if locked by default, and should be relocked on the appropriate tick
	bool	m_bDefaultClosed ;		// true if closed by default, and should be relocked on the appropriate tick

	bool	m_bCurrentlyLocked ;	// true if currently locked
	bool	m_bCurrentlyClosed ;	// true if currently closed

	string	m_sMyExitMessage ;		// text you see when you go through
	string	m_sOtherExitMessage ;	// text others see when you go through
	string	m_sMyEnterMessage ;		// text others see when you enter the room this exit points to
	string	m_sOtherEnterMessage ;	// text others see when you enter the room this exit points to

	unsigned long	m_ulTemporaryDest ;		// when initially loading from file, we need to use only the Room ID.  The pointer is resolved later.

public:
	CPsyMudRoomExit() ;
	CPsyMudRoomExit(const CPsyMudRoomExit& copy) ;
	~CPsyMudRoomExit() 	{}

	CPsyMudRoomExit& CPsyMudRoomExit::operator = (const CPsyMudRoomExit& rhs) ;


	inline void SetTemporaryDest (const unsigned long l){ m_ulTemporaryDest = l ; }
	inline const unsigned long GetTemporaryDest() const	{ return m_ulTemporaryDest ; }

	inline void SetDirectionName (const string& s)		{ m_sDirectionName = s ; }
	inline const string GetDirectionName() const		{ return m_sDirectionName ; }

	inline void SetDescription (const string& s)		{ m_sDescription = s ; }
	inline const string GetDescription() const			{ return m_sDescription ; }

	//inline void SetDestination (CPsyMudRoom * rm)		{ m_Destination = rm ; }	// removed due to circular dependency
	void 	SetDestination (CPsyMudObject * rm) ;
	//inline CPsyMudRoom * GetDestination() const		{ return m_Destination ; }
	inline CPsyMudObject * GetDestination() const		{ return m_Destination ; }

	inline void SetMyExitMessage (const string& msg)		{ m_sMyExitMessage = msg ; }
	inline const string GetMyExitMessage() const		{ return m_sMyExitMessage ; }

	inline void SetOtherExitMessage (const string& msg)	{ m_sOtherExitMessage = msg ; }
	inline const string GetOtherExitMessage() const		{ return m_sOtherExitMessage ; }

	inline void SetMyEnterMessage (const string& msg)	{ m_sMyEnterMessage = msg ; }
	inline const string GetMyEnterMessage() const		{ return m_sMyEnterMessage ; }

	inline void SetOtherEnterMessage (const string& msg){ m_sOtherEnterMessage = msg ; }
	inline const string GetOtherEnterMessage() const	{ return m_sOtherEnterMessage ; }

} ;

#endif	//#define CPSYMUDROOMEXIT_H
