//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================



#ifndef	CPSYMUDUNIVERSE_H
#define	CPSYMUDUNIVERSE_H

#include <string>
#include <exception>
//#include <vector>
#include <map>
#include "CPsyMudLog.h"
#include "CPsyMudObject.h"
#include "CPsyMudRoom.h"
#include "CPsyMudItem.h"

//#define UNIVERSE_LOGIN_BANNER		"banner.login"
//#define UNIVERSE_LOGOUT_BANNER		"banner.logout"

class CPsyMudUniverseException : public exception
{
private:
	string sLocation ;		// locatation of error
	string sErrorString ;	// description of exception

public:
	CPsyMudUniverseException(char * szLocation, char * szWhat)
				: sLocation(szLocation), sErrorString(szWhat) {}

	virtual const char *what() const
	{
		return sErrorString.c_str() ;
	}
	
	const char *where() const
	{
		return sLocation.c_str() ;
	}

} ;



class CPsyMudUniverse
{
private:
	string			m_sLoginBanner ;			// banner which is displayed on login
	string			m_sLogoutBanner ;			// banner which is displayed on logout

	string			m_sUniverseDirectory ;		// location where universe files are kept

	map<unsigned long, CPsyMudRoom *> m_Rooms ;		// list of rooms
	map<unsigned long, CPsyMudItem *> m_ItemTemplates ;		// list of item templates, not instances


	void 	LoadLoginBanner(const string sFileName) ;					// loads login banners
	void 	LoadLogoutBanner(const string sFileName) ;					// loads logout banner

	void 	LoadWorlds(const string sFileName) ;
	void 	LoadZones(const string sFileName) ;
	void	LoadRooms(const string sFileName) ;
	void 	LoadNPCs(const string sFileName) ;
	void 	LoadItems(const string sFileName) ;
	void 	LoadScripts(const string sFileName) ;

	void 	ResolveIDsIntoPointers() ;					// after loading, this changes all IDs to pointers (exits etc)

public:
	CPsyMudUniverse()	 ;
	~CPsyMudUniverse()	{ cerr << "(DEBUG) : CPsyMudUniverse destructor called." << endl ; }

	void		SetDirectory (const char * szDirectory) ;
	void		LoadUniverse() throw(CPsyMudUniverseException) ;

	CPsyMudRoom *	GetRoomByNumber(unsigned long ulRoomNumber) ;
	CPsyMudItem *	GetItemTemplateByNumber(unsigned long ulItemNumber) ;

	CPsyMudItem *	LoadItemIntoRoom(unsigned long ulItemNumber, unsigned long ulRoomNumber) ;

	CPsyMudRoom *	GetStartRoom(int iWorldID) 	{ return GetRoomByNumber(5000) ; }		// TEMP!!!!!!!!!

	const string	GetLoginBanner() const		{ return m_sLoginBanner ; }
	const string	GetLogoutBanner() const		{ return m_sLogoutBanner ; }


} ;


#endif //#define	CPSYMUDUNIVERSE_H
