//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================

#include "color.h"

string& ConvertColorCodesToANSIColor(const string &sSource, string &sConvertedString)
{
	string 	sCurrentCode ;
	bool 	bInColorCode = false ;
	
	for (string::const_iterator p = sSource.begin() ; p < sSource.end() ; p++)
	{
		if (*p == '&')
		{
			bInColorCode = true ;
		}

		if (bInColorCode)
		{
			sCurrentCode += *p ;

			// try to match with a color code
			if (sCurrentCode.length() == 2)
			{
				if (sCurrentCode != "&B")
				{
					if (sCurrentCode == COLOR_NORMAL)
						sConvertedString += "\x1B[0m" ;
					else if (sCurrentCode == COLOR_DARK_RED)
						sConvertedString += "\x1B[0;31m" ;
					else if (sCurrentCode == COLOR_DARK_GREEN)
						sConvertedString += "\x1B[0;32m" ;
					else if (sCurrentCode == COLOR_DARK_YELLOW)
						sConvertedString += "\x1B[0;33m" ;
					else if (sCurrentCode == COLOR_DARK_BLUE)
						sConvertedString += "\x1B[0;34m" ;
					else if (sCurrentCode == COLOR_DARK_MAGENTA)
						sConvertedString += "\x1B[0;35m" ;
					else if (sCurrentCode == COLOR_DARK_CYAN)
						sConvertedString += "\x1B[0;36m" ;
					else if (sCurrentCode == COLOR_DARK_WHITE)
						sConvertedString += "\x1B[0;37m" ;
					else if (sCurrentCode == COLOR_DARK_GRAY)
						sConvertedString += "\x1B[0;38m"	;	// not correct color code
					else
						sConvertedString += sCurrentCode ;		// not a color code

					bInColorCode = false ;
					sCurrentCode = "" ;
				}
			}
			else if  (sCurrentCode.length() == 4)
			{
				if (sCurrentCode == COLOR_BRIGHT_NORMAL)
					sConvertedString += "\x1B[0m" ;
				else if (sCurrentCode == COLOR_BRIGHT_RED)
					sConvertedString += "\x1B[1;31m" ;
				else if (sCurrentCode == COLOR_BRIGHT_GREEN)
					sConvertedString += "\x1B[1;32m" ;
				else if (sCurrentCode == COLOR_BRIGHT_YELLOW)
					sConvertedString += "\x1B[1;33m" ;
				else if (sCurrentCode == COLOR_BRIGHT_BLUE)
					sConvertedString += "\x1B[1;34m" ;
				else if (sCurrentCode == COLOR_BRIGHT_MAGENTA)
					sConvertedString += "\x1B[1;35m" ;
				else if (sCurrentCode == COLOR_BRIGHT_CYAN)
					sConvertedString += "\x1B[1;36m" ;
				else if (sCurrentCode == COLOR_BRIGHT_WHITE)
					sConvertedString += "\x1B[1;37m" ;
				else if (sCurrentCode == COLOR_BRIGHT_GRAY)
					sConvertedString += "\x1B[1;38m" ;
				else
					sConvertedString += sCurrentCode ;		// not a color code

				bInColorCode = false ;
				sCurrentCode = "" ;
			}
		}
		else
		{
			// not in color code, so just append character to string
			sConvertedString += *p ;
		}
	}
	
	if (sCurrentCode != "")
	{
		// handle partial trailing codes
		sConvertedString += sCurrentCode ;
	}
	
	// always make sure line ends with normal color, that way no one bleeds
	// their colors around onto other lines

	sConvertedString += "\x1B[0m" ;

	return sConvertedString ;
}
