//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================

#include "dice.h"
#include <stdio.h>

void ConvertDiceStringToStruct(const string& sDice, CPsyMudDiceDescriptor& Dice)
{
	int 			i ;

	static string sTemp  ;

		// find the number of dice

	for (sTemp = "", i = 0 ; i < sDice.length() && sDice[i] != 'd' && sDice[i] != 'D' ; i++)
	{
		sTemp += sDice[i] ;
	}
	Dice.NumDice = atoi(sTemp.c_str()) ;

	i+= 1 ;	// skip past the 'd'

	for (sTemp = "" ; i < sDice.length() && sDice[i] != '+' && sDice[i] != '-' ; i++)
	{
		sTemp += sDice[i] ;
	}
	Dice.NumSides = atoi(sTemp.c_str()) ;

	for (sTemp = "" ; i < sDice.length() ; i++)
	{
		sTemp += sDice[i] ;
	}
	Dice.Modifier = atoi(sTemp.c_str()) ;
}

// Takes dice description in the 2d8+5 style
const signed long int RollDice(const string& sDice)
{
	
	static CPsyMudDiceDescriptor Dice ;

	ConvertDiceStringToStruct(sDice, Dice) ;

	return RollDice(Dice) ;	
}

const signed long int RollDice(const CPsyMudDiceDescriptor& Dice)
{
	return RollDice(Dice.NumDice, Dice.NumSides, Dice.Modifier) ;
}


// Takes dice description in a computer-friendly broken-out format.
// typically this is only called by the other roll functions
const signed long int RollDice(const unsigned int iNumDice, const unsigned int iNumSides, const signed int iModifier)
{
	long 	lRand ;
	long	lTotalRoll = 0 ;

	if (iNumDice == 0 || iNumSides == 0)
	{
		// cannot roll 0 dice, or a 0-sided die
		return 0 ;
	}
	else
	{
		srandom(time(NULL)) ;

		for (int i = 0 ; i < iNumDice ; i++)
		{
			lTotalRoll += (random() % iNumSides) ;
		}

		lTotalRoll += iModifier ;

		return lTotalRoll ;
	}
}