//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================

#include "fileutil.h"
#include "stringutil.h"
#include <fstream>

string ExtensionFromFileName(const string& sFileName)
{
	string sExtension ;

	// iterate through the string from back to front until we find the period
	for (string::const_reverse_iterator p = sFileName.rbegin() ; p < sFileName.rend() ; p++)
	{
		if ((*p) == '.')
		{
			// found the period.  end here
			return sExtension ;
		}
		else
		{
			// because we go back to front it is *p + ext, not ext += *p
			sExtension = (*p) + sExtension ;
		}
	}

	// no period, so there is no extension in the file name
	return string("") ;
}

string ValFromKeyValPair(const string& sKeyValPair)
{
	string sKey ;
	string sVal ;

	SplitString(sKey, sVal, sKeyValPair, '=') ;

	return sVal ;
}


string ReadEverythingBetweenTags(ifstream& ifs, const char * szStartTag, const char * szEndTag)
{
	static char buf [1024+1] ;

	string 	sData ;
	long	lPos ;

	while(!ifs.eof())
	{
		// -1 so we retain at least the trailing nullchar

		ifs.getline(buf, sizeof(buf)-1, '>') ;
		
		sData += buf ;

		// need to pick up the end part of the tag, the ">" we told ifs to ignore
		sData += '>' ;

		lPos = sData.rfind(szEndTag) ;

		if (lPos < sData.length())
		{
			// found, since we used the '>' as the EOL character, we shouldn't have overshot the tag
			if ((lPos + strlen(szEndTag)) != sData.length())
			{
				cerr << "DEBUG: Bug or invalid file format." << endl ;
				cerr << "   szStartTag = " << szStartTag << endl ;
				cerr << "   szEndTag = " << szEndTag << endl ;
				cerr << "   lPos = " << lPos << endl ;
				cerr << "   sData.length() = " << sData.length() << endl ;

				return string("") ;
			}
		}
		memset(buf, 0, sizeof(buf)) ;		
	}

	return sData ;
}

