//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================



#include <iostream>
#include "CPsyMudApp.h"
#include "CPsyMudLog.h"
#include "sighand.h"

int main(int argc, char **argv)
{
	int 			iPos = 1;						// position in command line args
	char * 			szLogName = "log/psymud.log" ;	// log name
	char * 			szDataDirectory = "universe" ;	// universe data directory name
	bool 			bRestricted = false ;			// true if restricting to old players
	unsigned int 	uiPort = 2440 ;					// port number

	//cerr << "main : About to loop through args." << endl ;

	while ((iPos < argc) && (*(argv[iPos]) == '-'))
	{
		switch (*(argv[iPos] + 1))
		//switch (*(argv[iPos + 1]))
		{
			case 'l':
				if (*(argv[iPos] + 2))
					szLogName = argv[iPos] + 2 ;
				else if (++iPos < argc)
					szLogName = argv[iPos] ;
				else
				{
					cerr << "System Error : No log file name was specified after -l option." << endl ;
					return 1;
		   	}
				//cerr << "main : szLogName set to \"" << szLogName << "\" from -l option" << endl ;
	    	break;

			case 'r':
				bRestricted = true ;
				cerr << "System Info : Restricting game - no new players will be allowed." ;
				break;

    	case 'u':
				if (*(argv[iPos] + 2))
					szDataDirectory = argv[iPos] + 2;
				else if (++iPos < argc)
					szDataDirectory = argv[iPos];
				else
				{
					cerr << "System Error : No Directory name was specified after option -d option." << endl ;
					return 1;
				}
				//cerr << "main : szDataDirectory set to \"" << szDataDirectory << "\" from -d option" << endl ;
		   	break;

		  case 'h':
				cerr << "Usage: " << argv[0] << " [-h] [-l logname] [-r] [-u pathname] [port number]" << endl ;
              	cerr << "  -h             Display this help." << endl ;
				cerr << "  -l <file>      Write log to specified log file instead of default." << endl ;
              	cerr << "  -r             Restrict mud to existing players." << endl ;
              	cerr << "  -u <directory> Set location of universe data to specified directory." << endl ;
				return 0 ;

			default:
				cerr << "System Error : Unknown option -" << *(argv[iPos] + 1) << " in command line." << endl ;       			
				return 1 ;
		}

		iPos++;
	}

	//cerr << "main : Done looping through args." << endl ;

	if (iPos < argc)
	{
		if (!isdigit(*argv[iPos]))
		{
			cerr << "Usage: " << argv[0] << " [-h] [-l logname] [-r] [-u pathname] [port number]" << endl ;
			return 1 ;
		}
		else if ((uiPort = atoi(argv[iPos])) <= 1024)
		{
			cerr << "System Error : Ports <= 1024 are privledged ports. " << uiPort << " is illegal." << endl ;
			return 1 ;
		}
	}

		// set startup parameters
	g_Mud.SetPort(uiPort) ;
	g_Mud.SetRestricted(bRestricted) ;
	g_Mud.SetUniverseDirectory(szDataDirectory) ;

	//cerr << "main : About to run." << endl ;

	SetupSignalHandlers() ;

		// run the application
	g_Mud.Run() ;	

	return 0 ;
}