//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================


#include "sighand.h"
#include <stdlib.h>
#include "CPsyMudLog.h"

void SignalHandlerForChildren(int sig)
{
	// TO DO: Clean up zombies if we end up forking any
}


void SignalHandlerForHUP(int sig)
{
	g_Log.Log(CPsyMudLog::logError, "Received SIGHUP.  Shutting down.") ;
	// shut down
	// object desctructors will shut down everything nicely
	exit(1) ;
}

void SignalHandlerForINT(int sig)
{
	g_Log.Log(CPsyMudLog::logError, "Received SIGINT.  Shutting down.") ;
	// shut down
	// object desctructors will shut down everything nicely
	exit(1) ;
}

void SignalHandlerForTERM(int sig)
{
	g_Log.Log(CPsyMudLog::logError, "Received SIGTERM.  Shutting down.") ;
	// shut down
	// object desctructors will shut down everything nicely
	exit(1) ;
}

void SetupSingleSignalHandler(int SignalNumber, void (*func)(int))
{
	struct sigaction act, oact ;
	
	act.sa_handler = func ;
	sigemptyset(&act.sa_mask) ;
	act.sa_flags = 0 ;

	if (sigaction(SignalNumber, &act, &oact) < 0)
	{
	//	return (SIG_ERR) ;
	}
	else
	{
	//	return (oact.sa_handler) ;
	}
}

void SetupSignalHandlers()
{
	SetupSingleSignalHandler(SIGHUP, SignalHandlerForHUP) ;
	SetupSingleSignalHandler(SIGCHLD, SignalHandlerForChildren) ;
	SetupSingleSignalHandler(SIGINT, SignalHandlerForINT) ;
	SetupSingleSignalHandler(SIGTERM, SignalHandlerForTERM) ;	
	SetupSingleSignalHandler(SIGPIPE, SIG_IGN) ;	
	SetupSingleSignalHandler(SIGALRM, SIG_IGN) ;	
}

