//=======================================================================
// This file is a component of PsyMUD written by Peter M. Brown
// copyright            : (C) 2000 by Peter M Brown
// email                : pmbrown@cablespeed.com
//=======================================================================
//                                                                          
// This program is free software; you can redistribute it and/or modify  
// it under the terms of the GNU General Public License as published by  
// the Free Software Foundation; either version 2 of the License, or     
// (at your option) any later version. Pleae keep all headers and 
// documentation (including development credits) intact.
//                                                                          
//=======================================================================


#include "stringutil.h"

#define IsWhiteSpace(ch) (ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r')

string StripLeadingBlanks(const string& sSource)
{
	string::const_iterator p ;

	string::const_iterator pCopy ;

	string sResult ;

	for (p = sSource.begin() ; p != sSource.end() && IsWhiteSpace(*p) ; p++)
		;

	// construct a string that is between the two iterators
	for (pCopy = p ; pCopy < sSource.end() ; pCopy++)
	{
		sResult += *pCopy ;
	}

	return sResult ;
}

string StripTrailingBlanks(const string& sSource)
{
	string::const_reverse_iterator p ;
	string::const_reverse_iterator pCopy ;

	string sResult ;

	for (p = sSource.rbegin() ; p < sSource.rend() && IsWhiteSpace(*p) ; p++)
		;

	// construct a string that is between the two iterators
	// note that we're going in reverse here
	for (pCopy = p ; pCopy < sSource.rend() ; pCopy++)
	{
		sResult = *pCopy + sResult ;
	}

	return sResult ;
}


// this will chunk out everything that is between the specified tags
string RemoveEverythingBetweenTags(string& str, const char * szStartTag, const char * szEndTag)
{
	unsigned int		iStart ;
	unsigned int		iEnd ;

	iStart = str.find(szStartTag) ;

	if (iStart < str.length())
	{
		iEnd = str.find(szEndTag, iStart) ;

		if (iEnd < str.length())
		{
			string sResult ;
			
			sResult = str.substr(iStart+strlen(szStartTag), iEnd-(iStart+strlen(szStartTag))) ;

			// this effectively removes the stuff we just parsed out
			// from the parent string.
			str = str.substr(0, iStart) + str.substr(iEnd + strlen(szEndTag)) ;

			//cerr << "DEBUG: RemoveEverythingBetweenTags " << szStartTag << " and " << szEndTag << endl ;
			//cerr << "  sResult = " << sResult << endl ;
			//cerr << "  str = " << str << endl ;

			return StripLeadingBlanks(StripTrailingBlanks(sResult)) ;
		}
		else
		{
			// syntax problem.  should probably raise an exception
			// start tag was found but end tag was not
			cerr << "DEBUG: End tag \"" << szEndTag << "\" not found." << endl ;
			return string("") ;
		}
	}
	else
	{
		// start tag wasn't found
		//cerr << "DEBUG: Start tag \"" << szStartTag << "\" not found." << endl ;

		return string("") ;
	}
	
}



void SplitString(string &sLeftSide, string &sRightSide, const string &sSource, const char chSplitOn = ' ')
{
	string::const_iterator p ;

	sLeftSide = "" ;
	sRightSide = "" ;

		// skip over whitespace.  I can do this here because this function is specific to the
		// mud processing.  if it were truly general purpose I'd think twice about doing this
	for (p = sSource.begin() ; p != sSource.end() && (*p == ' ' || *p == '\n' || *p == '\r' || *p == '\t') ; p++)
		;

		// pull off first word before the split
	for (/* pick up where we left off */ ; p != sSource.end() && *p != chSplitOn ; p++)
	{
		sLeftSide += *p ;
	}

		// pick up the rest of the sentence
	if (p != sSource.end())
	{
		for ( p += 1 ; p != sSource.end() ; p++)
		{
			sRightSide += *p ;
		}	
	}	
}
